@@ -27,12 +27,16 @@ class MyHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 		       acceptable.get(leaf, None) or \
 		       acceptable.get('*', None)
 
+		# (don't use a symlink as they don't work on Windows)
+		if leaf == 'latest.xml':
+			leaf = 'Hello.xml'
+
 		if not resp:
 			self.send_error(404, "Expected %s; got %s" % (next_step, parsed.path))
 		elif parsed.path.startswith('/key-info/'):
 			self.send_response(200)
 			self.end_headers()
-			self.wfile.write('<key-lookup/>')
+			self.wfile.write('<key-lookup><item vote="good">Approved for testing</item></key-lookup>')
 			self.wfile.close()
 		elif os.path.exists(leaf) and not isinstance(resp, Give404):
 			self.send_response(200)
