@@ -2,13 +2,13 @@
 # -*- coding: utf-8 -*-
 import basetest
 from basetest import BaseTest, empty_feed
-import sys, os, locale
+import sys, os
 from xml.dom import minidom
 import unittest
 from StringIO import StringIO
 
 sys.path.insert(0, '..')
-from zeroinstall.injector import model, qdom, iface_cache, namespaces
+from zeroinstall.injector import model, qdom, namespaces
 
 mydir = os.path.dirname(__file__)
 
@@ -97,7 +97,8 @@ class TestModel(BaseTest):
 		local_path = os.path.join(mydir, 'Local.xml')
 		dom = qdom.parse(open(local_path))
 		feed = model.ZeroInstallFeed(dom, local_path = local_path)
-		self.assertEquals("Local feed", feed.summary)
+		# (defaults to en-US if no language is set in the locale)
+		self.assertEquals("Local feed (English)", feed.summary)
 		self.assertEquals("English", feed.description)
 
 		self.assertEquals(4, len(feed.summaries))
@@ -109,9 +110,13 @@ class TestModel(BaseTest):
 			self.assertEquals("Fuente local", feed.summary)
 			self.assertEquals(u"Español", feed.description)
 
+			basetest.test_locale = ('en_GB', 'UTF8')
+
+			self.assertEquals("Local feed (English GB)", feed.summary)
+
 			basetest.test_locale = ('fr_FR', 'UTF8')
 
-			self.assertEquals("Local feed", feed.summary)
+			self.assertEquals("Local feed (English)", feed.summary)
 			self.assertEquals("English", feed.description)
 		finally:
 			basetest.test_locale = (None, None)
@@ -251,14 +256,14 @@ class TestModel(BaseTest):
 		try:
 			f = model.Feed('http://feed', arch = 'i386', user_override = True)
 			assert False
-		except model.SafeException, ex:
+		except model.SafeException as ex:
 			assert 'Malformed arch' in str(ex)
 	
 	def testCanonical(self):
 		try:
 			model.canonical_iface_uri('http://foo')
 			assert False
-		except model.SafeException, ex:
+		except model.SafeException as ex:
 			assert 'Missing /' in str(ex)
 
 		self.assertEquals('http://foo/',
@@ -266,7 +271,7 @@ class TestModel(BaseTest):
 		try:
 			model.canonical_iface_uri('bad-name')
 			assert False
-		except model.SafeException, ex:
+		except model.SafeException as ex:
 			assert 'Bad interface name' in str(ex)
 	
 	def testVersions(self):
@@ -285,7 +290,7 @@ class TestModel(BaseTest):
 			try:
 				pv(v)
 				assert False
-			except model.SafeException, ex:
+			except model.SafeException:
 				pass
 		invalid('.')
 		invalid('hello')
