@@ -5,17 +5,17 @@ import sys, os
 import unittest
 
 sys.path.insert(0, '..')
-from zeroinstall.injector import selections, model, reader, policy, iface_cache, namespaces, qdom
+from zeroinstall.injector import selections, model, policy, namespaces, qdom
 
 mydir = os.path.dirname(os.path.abspath(__file__))
 
 class TestSelections(BaseTest):
 	def testSelections(self):
-		p = policy.Policy('http://foo/Source.xml', src = True)
-		source = iface_cache.iface_cache.get_interface('http://foo/Source.xml')
-		compiler = iface_cache.iface_cache.get_interface('http://foo/Compiler.xml')
-		reader.update(source, 'Source.xml')
-		reader.update(compiler, 'Compiler.xml')
+		p = policy.Policy('http://foo/Source.xml', src = True, config = self.config)
+		source = self.config.iface_cache.get_interface('http://foo/Source.xml')
+		compiler = self.config.iface_cache.get_interface('http://foo/Compiler.xml')
+		self.import_feed(source.uri, 'Source.xml')
+		self.import_feed(compiler.uri, 'Compiler.xml')
 
 		p.freshness = 0
 		p.network_use = model.network_full
@@ -52,10 +52,23 @@ class TestSelections(BaseTest):
 			self.assertEquals(1, len(sels[1].dependencies))
 			dep = sels[1].dependencies[0]
 			self.assertEquals('http://foo/Compiler.xml', dep.interface)
-			self.assertEquals(1, len(dep.bindings))
+			self.assertEquals(3, len(dep.bindings))
+			self.assertEquals('bin', dep.bindings[0].insert)
+			self.assertEquals('PATH', dep.bindings[0].name)
+			self.assertEquals('prepend', dep.bindings[0].mode)
+			assert dep.bindings[0].separator in ';:'
+
+			self.assertEquals('bin', dep.bindings[1].value)
+			self.assertEquals('NO_PATH', dep.bindings[1].name)
+			self.assertEquals(',', dep.bindings[1].separator)
+
+			self.assertEquals('bin', dep.bindings[2].insert)
+			self.assertEquals('BINDIR', dep.bindings[2].name)
+			self.assertEquals('replace', dep.bindings[2].mode)
+
 			self.assertEquals(["sha1=345"], sels[0].digests)
 
-		s1 = selections.Selections(p)
+		s1 = p.solver.selections
 		s1.selections['http://foo/Source.xml'].attrs['http://namespace foo'] = 'bar'
 		assertSel(s1)
 
@@ -69,9 +82,10 @@ class TestSelections(BaseTest):
 	def testLocalPath(self):
 		# 0launch --get-selections Local.xml
 		iface = os.path.join(mydir, "Local.xml")
-		p = policy.Policy(iface)
+		p = policy.Policy(iface, config = self.config)
 		p.need_download()
-		s1 = selections.Selections(p)
+		assert p.ready
+		s1 = p.solver.selections
 		xml = s1.toDOM().toxml("utf-8")
 
 		# Reload selections and check they're the same
@@ -82,7 +96,7 @@ class TestSelections(BaseTest):
 		assert not s2.selections[iface].digests, s2.selections[iface].digests
 
 		# Add a newer implementation and try again
-		feed = iface_cache.iface_cache.get_feed(iface)
+		feed = self.config.iface_cache.get_feed(iface)
 		impl = model.ZeroInstallImplementation(feed, "foo bar=123", local_path = None)
 		impl.version = model.parse_version('1.0')
 		impl.commands["run"] = model.Command(qdom.Element(namespaces.XMLNS_IFACE, 'command', {'path': 'dummy'}), None)
@@ -90,7 +104,7 @@ class TestSelections(BaseTest):
 		feed.implementations = {impl.id: impl}
 		assert p.need_download()
 		assert p.ready, p.solver.get_failure_reason()
-		s1 = selections.Selections(p)
+		s1 = p.solver.selections
 		xml = s1.toDOM().toxml("utf-8")
 		root = qdom.parse(StringIO(xml))
 		s2 = selections.Selections(root)
@@ -102,19 +116,19 @@ class TestSelections(BaseTest):
 
 	def testCommands(self):
 		iface = os.path.join(mydir, "Command.xml")
-		p = policy.Policy(iface)
+		p = policy.Policy(iface, config = self.config)
 		p.need_download()
 		assert p.ready
 
-		impl = p.solver.selections[iface_cache.iface_cache.get_interface(iface)]
+		impl = p.solver.selections[self.config.iface_cache.get_interface(iface)]
 		assert impl.id == 'c'
 		assert impl.main == 'runnable/missing'
 
 		dep_impl_uri = impl.commands['run'].requires[0].interface
-		dep_impl = p.solver.selections[iface_cache.iface_cache.get_interface(dep_impl_uri)]
+		dep_impl = p.solver.selections[self.config.iface_cache.get_interface(dep_impl_uri)]
 		assert dep_impl.id == 'sha1=256'
 
-		s1 = selections.Selections(p)
+		s1 = p.solver.selections
 		assert s1.commands[0].path == 'runnable/missing'
 		xml = s1.toDOM().toxml("utf-8")
 		root = qdom.parse(StringIO(xml))
@@ -131,5 +145,22 @@ class TestSelections(BaseTest):
 		dep_impl = s2.selections[dep_impl_uri]
 		assert dep_impl.id == 'sha1=256'
 
+	def testOldCommands(self):
+		command_feed = os.path.join(mydir, 'old-selections.xml')
+		with open(command_feed) as stream:
+			s1 = selections.Selections(qdom.parse(stream))
+		self.assertEquals("run", s1.command)
+		self.assertEquals(2, len(s1.commands))
+		self.assertEquals("bin/java", s1.commands[1].path)
+
+		xml = s1.toDOM().toxml("utf-8")
+		root = qdom.parse(StringIO(xml))
+		s2 = selections.Selections(root)
+
+		self.assertEquals("run", s2.command)
+		self.assertEquals(2, len(s2.commands))
+		self.assertEquals("bin/java", s2.commands[1].path)
+
+
 if __name__ == '__main__':
 	unittest.main()
