@@ -4,9 +4,7 @@ import sys, os, locale
 import unittest
 
 sys.path.insert(0, '..')
-from zeroinstall.zerostore import Stores
-from zeroinstall.injector import solver, reader, arch, model
-from zeroinstall.injector.iface_cache import iface_cache
+from zeroinstall.injector import solver, arch
 
 import logging
 logger = logging.getLogger()
@@ -14,14 +12,15 @@ logger = logging.getLogger()
 
 class TestSolver(BaseTest):
 	def testSimple(self):
-		s = solver.DefaultSolver(model.network_full, iface_cache, Stores())
+		iface_cache = self.config.iface_cache
+		s = solver.DefaultSolver(self.config)
 
 		foo = iface_cache.get_interface('http://foo/Binary.xml')
-		reader.update(foo, 'Binary.xml')
+		self.import_feed(foo.uri, 'Binary.xml')
 		foo_src = iface_cache.get_interface('http://foo/Source.xml')
-		reader.update(foo_src, 'Source.xml')
+		self.import_feed(foo_src.uri, 'Source.xml')
 		compiler = iface_cache.get_interface('http://foo/Compiler.xml')
-		reader.update(compiler, 'Compiler.xml')
+		self.import_feed(compiler.uri, 'Compiler.xml')
 
 		binary_arch = arch.Architecture({None: 1}, {None: 1})
 		assert str(binary_arch).startswith("<Arch")
@@ -43,14 +42,16 @@ class TestSolver(BaseTest):
 		assert not s.details
 	
 	def testDetails(self):
-		s = solver.DefaultSolver(model.network_full, iface_cache, Stores())
+		iface_cache = self.config.iface_cache
+		s = solver.DefaultSolver(self.config)
 
-		foo = iface_cache.get_interface('http://foo/Binary.xml')
-		reader.update(foo, 'Binary.xml')
+		foo_binary_uri = 'http://foo/Binary.xml'
+		foo = iface_cache.get_interface(foo_binary_uri)
+		self.import_feed(foo_binary_uri, 'Binary.xml')
 		foo_src = iface_cache.get_interface('http://foo/Source.xml')
-		reader.update(foo_src, 'Source.xml')
+		self.import_feed(foo_src.uri, 'Source.xml')
 		compiler = iface_cache.get_interface('http://foo/Compiler.xml')
-		reader.update(compiler, 'Compiler.xml')
+		self.import_feed(compiler.uri, 'Compiler.xml')
 
 		binary_arch = arch.Architecture({None: 1}, {None: 1})
 		s.record_details = True
@@ -68,10 +69,11 @@ class TestSolver(BaseTest):
 		assert s.details[compiler] == [(compiler_impls['sha1=345'], None)]
 
 	def testRecursive(self):
-		s = solver.DefaultSolver(model.network_full, iface_cache, Stores())
+		iface_cache = self.config.iface_cache
+		s = solver.DefaultSolver(self.config)
 
 		foo = iface_cache.get_interface('http://foo/Recursive.xml')
-		reader.update(foo, 'Recursive.xml')
+		self.import_feed(foo.uri, 'Recursive.xml')
 
 		binary_arch = arch.Architecture({None: 1}, {None: 1})
 		s.record_details = True
@@ -84,12 +86,13 @@ class TestSolver(BaseTest):
 		assert s.details[foo] == [(foo_impls['sha1=abc'], None)]
 		
 	def testMultiArch(self):
-		s = solver.DefaultSolver(model.network_full, iface_cache, Stores())
+		iface_cache = self.config.iface_cache
+		s = solver.DefaultSolver(self.config)
 
 		foo = iface_cache.get_interface('http://foo/MultiArch.xml')
-		reader.update(foo, 'MultiArch.xml')
+		self.import_feed(foo.uri, 'MultiArch.xml')
 		lib = iface_cache.get_interface('http://foo/MultiArchLib.xml')
-		reader.update(lib, 'MultiArchLib.xml')
+		self.import_feed(lib.uri, 'MultiArchLib.xml')
 
 		# On an i686 system we can only use the i486 implementation
 
@@ -124,7 +127,8 @@ class TestSolver(BaseTest):
 		assert 'ppc' not in other.machine_ranks
 	
 	def testRanking(self):
-		s = solver.DefaultSolver(model.network_full, iface_cache, Stores())
+		iface_cache = self.config.iface_cache
+		s = solver.DefaultSolver(self.config)
 		ranking = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'Ranking.xml')
 		iface = iface_cache.get_interface(ranking)
 
@@ -144,12 +148,13 @@ class TestSolver(BaseTest):
 			selected)
 
 	def testLangs(self):
+		iface_cache = self.config.iface_cache
 		try:
 			locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
 
-			s = solver.DefaultSolver(model.network_full, iface_cache, Stores())
+			s = solver.DefaultSolver(self.config)
 			iface = iface_cache.get_interface('http://foo/Langs.xml')
-			reader.update(iface, 'Langs.xml')
+			self.import_feed(iface.uri, 'Langs.xml')
 
 			# 1 is the oldest, but the only one in our language
 			binary_arch = arch.get_architecture(None, 'arch_1')
@@ -203,5 +208,10 @@ class TestSolver(BaseTest):
 		finally:
 			locale.setlocale(locale.LC_ALL, '')
 
+	def testDecideBug(self):
+		s = solver.DefaultSolver(self.config)
+		watch_xml = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'watchdog.xml')
+		s.solve(watch_xml, arch.get_architecture(None, None), command_name = 'test')
+
 if __name__ == '__main__':
 	unittest.main()
