@@ -6,6 +6,7 @@ import gtk, pango
 import dialog
 
 import zeroinstall
+from zeroinstall import _
 from zeroinstall import support
 from zeroinstall.injector import selections
 
@@ -32,15 +33,16 @@ def report_bug(policy, iface):
 	if not policy.ready:
 		text += '  Failed to select all required implementations\n'
 
-	for chosen_iface in policy.implementation:
-		text += '\n  Interface: %s\n' % chosen_iface.uri
-		impl = policy.implementation[chosen_iface]
+	for chosen_iface_uri, impl in policy.solver.selections.selections.iteritems():
+		text += '\n  Interface: %s\n' % chosen_iface_uri
 		if impl:
-			text += '    Version: %s\n' % impl.get_version()
-			if impl.feed.url != chosen_iface.uri:
-				text += '  From feed: %s\n' % impl.feed.url
+			text += '    Version: %s\n' % impl.version
+			feed_url = impl.attrs['from-feed']
+			if feed_url != chosen_iface_uri:
+				text += '  From feed: %s\n' % feed_url
 			text += '         ID: %s\n' % impl.id
 		else:
+			chosen_iface = policy.config.iface_cache.get_interface(chosen_iface_uri)
 			impls = policy.solver.details.get(chosen_iface, None)
 			if impls:
 				best, reason = impls[0]
@@ -56,7 +58,7 @@ def report_bug(policy, iface):
 		text += '\nSystem without uname()\n'
 
 	if policy.solver.ready:
-		sels = selections.Selections(policy)
+		sels = policy.solver.selections
 		text += "\n" + sels.toDOM().toprettyxml(encoding = 'utf-8')
 
 	reporter = BugReporter(policy, iface, text)
@@ -189,7 +191,7 @@ class BugReporter(dialog.Dialog):
 			return
 
 		from zeroinstall.injector import selections
-		sels = selections.Selections(self.policy)
+		sels = self.policy.solver.selections
 		doc = sels.toDOM()
 
 		self.hide()
