@@ -3,10 +3,11 @@
 
 import gobject
 
+from zeroinstall import _
 from zeroinstall.support import tasks
 from zeroinstall.injector import handler, download
 
-version = '0.51'
+version = '1.2'
 
 class GUIHandler(handler.Handler):
 	dl_callbacks = None		# Download -> [ callback ]
@@ -26,7 +27,7 @@ class GUIHandler(handler.Handler):
 	def downloads_changed(self):
 		if self.monitored_downloads and self.pulse is None:
 			def pulse():
-				self.mainwindow.update_download_status()
+				self.mainwindow.update_download_status(only_update_visible = True)
 				return True
 			pulse()
 			self.pulse = gobject.timeout_add(200, pulse)
@@ -41,7 +42,7 @@ class GUIHandler(handler.Handler):
 				self.mainwindow.update_download_status()
 
 	def impl_added_to_store(self, impl):
-		self.mainwindow.update_download_status()
+		self.mainwindow.update_download_status(only_update_visible = True)
 
 	@tasks.async
 	def _switch_to_main_window(self, reason):
@@ -67,7 +68,6 @@ class GUIHandler(handler.Handler):
 		yield self._switch_to_main_window(_('Need to confirm installation of distribution packages'))
 
 		from zeroinstall.injector.download import DownloadAborted
-		from zeroinstall.gtkui import gtkutils
 		import dialog
 		import gtk
 		box = gtk.MessageDialog(self.mainwindow.window,
