@@ -3,16 +3,17 @@
 
 import gtk, gobject, pango
 
+from zeroinstall import _, translation
 from zeroinstall.support import tasks, pretty_size
-from zeroinstall.injector.iface_cache import iface_cache
-from zeroinstall.injector import model
+from zeroinstall.injector import model, reader
 import properties
-from zeroinstall.gtkui.treetips import TreeTips
 from zeroinstall.gtkui.icon import load_icon
 from zeroinstall import support
-from logging import warn
+from logging import warn, info
 import utils
 
+ngettext = translation.ngettext
+
 def _stability(impl):
 	assert impl
 	if impl.user_stability is None:
@@ -24,49 +25,42 @@ def _stability(impl):
 ICON_SIZE = 20.0
 CELL_TEXT_INDENT = int(ICON_SIZE) + 4
 
-class InterfaceTips(TreeTips):
-	mainwindow = None
+def get_tooltip_text(mainwindow, interface, main_feed, model_column):
+	assert interface
+	if model_column == InterfaceBrowser.INTERFACE_NAME:
+		return _("Full name: %s") % interface.uri
+	elif model_column == InterfaceBrowser.SUMMARY:
+		if main_feed is None or not main_feed.description:
+			return _("(no description available)")
+		first_para = main_feed.description.split('\n\n', 1)[0]
+		return first_para.replace('\n', ' ')
+	elif model_column is None:
+		return _("Click here for more options...")
 
-	def __init__(self, mainwindow):
-		self.mainwindow = mainwindow
+	impl = mainwindow.policy.implementation.get(interface, None)
+	if not impl:
+		return _("No suitable version was found. Double-click "
+			 "here to find out why.")
 
-	def get_tooltip_text(self):
-		interface, model_column = self.item
-		assert interface
-		if model_column == InterfaceBrowser.INTERFACE_NAME:
-			return _("Full name: %s") % interface.uri
-		elif model_column == InterfaceBrowser.SUMMARY:
-			if not interface.description:
-				return None
-			first_para = interface.description.split('\n\n', 1)[0]
-			return first_para.replace('\n', ' ')
-		elif model_column is None:
-			return _("Click here for more options...")
+	if model_column == InterfaceBrowser.VERSION:
+		text = _("Currently preferred version: %(version)s (%(stability)s)") % \
+				{'version': impl.get_version(), 'stability': _stability(impl)}
+		old_impl = mainwindow.original_implementation.get(interface, None)
+		if old_impl is not None and old_impl is not impl:
+			text += '\n' + _('Previously preferred version: %(version)s (%(stability)s)') % \
+				{'version': old_impl.get_version(), 'stability': _stability(old_impl)}
+		return text
 
-		impl = self.mainwindow.policy.implementation.get(interface, None)
-		if not impl:
-			return _("No suitable implementation was found. Check the "
-				 "interface properties to find out why.")
+	assert model_column == InterfaceBrowser.DOWNLOAD_SIZE
 
-		if model_column == InterfaceBrowser.VERSION:
-			text = _("Currently preferred version: %(version)s (%(stability)s)") % \
-					{'version': impl.get_version(), 'stability': _stability(impl)}
-			old_impl = self.mainwindow.original_implementation.get(interface, None)
-			if old_impl is not None and old_impl is not impl:
-				text += '\n' + _('Previously preferred version: %(version)s (%(stability)s)') % \
-					{'version': old_impl.get_version(), 'stability': _stability(old_impl)}
-			return text
-
-		assert model_column == InterfaceBrowser.DOWNLOAD_SIZE
-
-		if self.mainwindow.policy.get_cached(impl):
-			return _("This version is already stored on your computer.")
-		else:
-			src = self.mainwindow.policy.fetcher.get_best_source(impl)
-			if not src:
-				return _("No downloads available!")
-			return _("Need to download %(pretty_size)s (%(size)s bytes)") % \
-					{'pretty_size': support.pretty_size(src.size), 'size': src.size}
+	if mainwindow.policy.get_cached(impl):
+		return _("This version is already stored on your computer.")
+	else:
+		src = mainwindow.policy.fetcher.get_best_source(impl)
+		if not src:
+			return _("No downloads available!")
+		return _("Need to download %(pretty_size)s (%(size)s bytes)") % \
+				{'pretty_size': support.pretty_size(src.size), 'size': src.size}
 
 class MenuIconRenderer(gtk.GenericCellRenderer):
 	def __init__(self):
@@ -143,6 +137,12 @@ if gtk.pygtk_version < (2, 8, 0):
 	gobject.type_register(IconAndTextRenderer)
 	gobject.type_register(MenuIconRenderer)
 
+def walk(model, it):
+	while it:
+		yield it
+		for x in walk(model, model.iter_children(it)): yield x
+		it = model.iter_next(it)
+
 class InterfaceBrowser:
 	model = None
 	root = None
@@ -157,6 +157,8 @@ class InterfaceBrowser:
 	SUMMARY = 3
 	DOWNLOAD_SIZE = 4
 	ICON = 5
+	BACKGROUND = 6
+	PROBLEM = 7
 
 	columns = [(_('Component'), INTERFACE_NAME),
 		   (_('Version'), VERSION),
@@ -165,22 +167,42 @@ class InterfaceBrowser:
 		   ('', None)]
 
 	def __init__(self, policy, widgets):
-		tips = InterfaceTips(self)
-
 		tree_view = widgets.get_widget('components')
+		tree_view.set_property('has-tooltip', True)
+		def callback(widget, x, y, keyboard_mode, tooltip):
+			x, y = tree_view.convert_widget_to_bin_window_coords(x, y)
+			pos = tree_view.get_path_at_pos(x, y)
+			if pos:
+				tree_view.set_tooltip_cell(tooltip, pos[0], pos[1], None)
+				path = pos[0]
+				try:
+					col_index = column_objects.index(pos[1])
+				except ValueError:
+					return False
+				else:
+					col = self.columns[col_index][1]
+					row = self.model[path]
+					iface = row[InterfaceBrowser.INTERFACE]
+					main_feed = self.policy.config.iface_cache.get_feed(iface.uri)
+					tooltip.set_text(get_tooltip_text(self, iface, main_feed, col))
+				return True
+			else:
+				return False
+		tree_view.connect('query-tooltip', callback)
 
 		self.policy = policy
 		self.cached_icon = {}	# URI -> GdkPixbuf
 		self.default_icon = tree_view.style.lookup_icon_set(gtk.STOCK_EXECUTE).render_icon(tree_view.style,
 			gtk.TEXT_DIR_NONE, gtk.STATE_NORMAL, gtk.ICON_SIZE_SMALL_TOOLBAR, tree_view, None)
 
-		self.model = gtk.TreeStore(object, str, str, str, str, gtk.gdk.Pixbuf)
+		self.model = gtk.TreeStore(object, str, str, str, str, gtk.gdk.Pixbuf, str, bool)
 		self.tree_view = tree_view
 		tree_view.set_model(self.model)
 
 		column_objects = []
 
 		text = gtk.CellRendererText()
+		coloured_text = gtk.CellRendererText()
 
 		for name, model_column in self.columns:
 			if model_column == InterfaceBrowser.INTERFACE_NAME:
@@ -198,6 +220,9 @@ class InterfaceBrowser:
 						pass
 					column = gtk.TreeViewColumn(name, text_ellip, text = model_column)
 					column.set_expand(True)
+				elif model_column == InterfaceBrowser.VERSION:
+					column = gtk.TreeViewColumn(name, coloured_text, text = model_column,
+								    background = InterfaceBrowser.BACKGROUND)
 				else:
 					column = gtk.TreeViewColumn(name, text, text = model_column)
 			tree_view.append_column(column)
@@ -207,28 +232,6 @@ class InterfaceBrowser:
 
 		selection = tree_view.get_selection()
 
-		def motion(tree_view, ev):
-			if ev.window is not tree_view.get_bin_window():
-				return False
-			pos = tree_view.get_path_at_pos(int(ev.x), int(ev.y))
-			if pos:
-				path = pos[0]
-				try:
-					col_index = column_objects.index(pos[1])
-				except ValueError:
-					tips.hide()
-				else:
-					col = self.columns[col_index][1]
-					row = self.model[path]
-					item = (row[InterfaceBrowser.INTERFACE], col)
-					if item != tips.item:
-						tips.prime(tree_view, item)
-			else:
-				tips.hide()
-
-		tree_view.connect('motion-notify-event', motion)
-		tree_view.connect('leave-notify-event', lambda tv, ev: tips.hide())
-
 		def button_press(tree_view, bev):
 			pos = tree_view.get_path_at_pos(int(bev.x), int(bev.y))
 			if not pos:
@@ -243,7 +246,7 @@ class InterfaceBrowser:
 				return True
 			if bev.button != 1 or bev.type != gtk.gdk._2BUTTON_PRESS:
 				return False
-			properties.edit(policy, self.model[path][InterfaceBrowser.INTERFACE])
+			properties.edit(policy, self.model[path][InterfaceBrowser.INTERFACE], self.compile, show_versions = True)
 		tree_view.connect('button-press-event', button_press)
 
 		tree_view.connect('destroy', lambda s: policy.watchers.remove(self.build_tree))
@@ -268,7 +271,7 @@ class InterfaceBrowser:
 			return self.cached_icon[iface.uri]
 		except KeyError:
 			# Try the on-disk cache
-			iconpath = iface_cache.get_icon_path(iface)
+			iconpath = self.policy.config.iface_cache.get_icon_path(iface)
 
 			if iconpath:
 				icon = load_icon(iconpath, ICON_SIZE, ICON_SIZE)
@@ -293,13 +296,13 @@ class InterfaceBrowser:
 							# Try to insert new icon into the cache
 							# If it fails, we'll be left with None in the cached_icon so
 							# we don't try again.
-							iconpath = iface_cache.get_icon_path(iface)
+							iconpath = self.policy.config.iface_cache.get_icon_path(iface)
 							if iconpath:
 								self.cached_icon[iface.uri] = load_icon(iconpath, ICON_SIZE, ICON_SIZE)
 								self.build_tree()
 							else:
 								warn("Failed to download icon for '%s'", iface)
-						except Exception, ex:
+						except Exception as ex:
 							import traceback
 							traceback.print_exc()
 							self.policy.handler.report_error(ex)
@@ -316,25 +319,37 @@ class InterfaceBrowser:
 		return None
 
 	def build_tree(self):
+		iface_cache = self.policy.config.iface_cache
+
 		if self.original_implementation is None:
 			self.set_original_implementations()
 
 		done = {}	# Detect cycles
 
+		sels = self.policy.solver.selections
+
 		self.model.clear()
-		parent = None
-		def add_node(parent, iface, command = None):
+		def add_node(parent, iface, commands, essential):
 			if iface in done:
 				return
 			done[iface] = True
 
+			main_feed = iface_cache.get_feed(iface.uri)
+			if main_feed:
+				name = main_feed.get_name()
+				summary = main_feed.summary
+			else:
+				name = iface.get_name()
+				summary = None
+
 			iter = self.model.append(parent)
 			self.model[iter][InterfaceBrowser.INTERFACE] = iface
-			self.model[iter][InterfaceBrowser.INTERFACE_NAME] = iface.get_name()
-			self.model[iter][InterfaceBrowser.SUMMARY] = iface.summary
+			self.model[iter][InterfaceBrowser.INTERFACE_NAME] = name
+			self.model[iter][InterfaceBrowser.SUMMARY] = summary
 			self.model[iter][InterfaceBrowser.ICON] = self.get_icon(iface) or self.default_icon
+			self.model[iter][InterfaceBrowser.PROBLEM] = False
 
-			sel = self.policy.solver.selections.selections.get(iface.uri, None)
+			sel = sels.selections.get(iface.uri, None)
 			if sel:
 				impl = sel.impl
 				old_impl = self.original_implementation.get(iface, None)
@@ -346,11 +361,14 @@ class InterfaceBrowser:
 				self.model[iter][InterfaceBrowser.DOWNLOAD_SIZE] = utils.get_fetch_info(self.policy, impl)
 
 				deps = sel.dependencies
-				if command:
-					deps += command.requires
+				for c in commands:
+					deps += sel.get_command(c).requires
 				for child in deps:
 					if isinstance(child, model.InterfaceDependency):
-						add_node(iter, iface_cache.get_interface(child.interface))
+						add_node(iter,
+							 iface_cache.get_interface(child.interface),
+							 child.get_required_commands(),
+							 child.importance == model.Dependency.Essential)
 					else:
 						child_iter = self.model.append(parent)
 						self.model[child_iter][InterfaceBrowser.INTERFACE_NAME] = '?'
@@ -358,25 +376,19 @@ class InterfaceBrowser:
 							_('Unknown dependency type : %s') % child
 						self.model[child_iter][InterfaceBrowser.ICON] = self.default_icon
 			else:
-				self.model[iter][InterfaceBrowser.VERSION] = _('(choose)')
-		commands = self.policy.solver.selections.commands
-		if commands:
-			for command in commands:
-				add_node(None, self.root, command)
-		else:
-			# Nothing could be selected
-			add_node(None, self.root, None)
+				self.model[iter][InterfaceBrowser.PROBLEM] = essential
+				self.model[iter][InterfaceBrowser.VERSION] = _('(problem)') if essential else _('(none)')
+		add_node(None, self.root, [sels.command], essential = True)
 		self.tree_view.expand_all()
-	
+
 	def show_popup_menu(self, iface, bev):
 		import bugs
-		import compile
 
 		have_source =  properties.have_source_for(self.policy, iface)
 
 		menu = gtk.Menu()
-		for label, cb in [(_('Show Feeds'), lambda: properties.edit(self.policy, iface)),
-				  (_('Show Versions'), lambda: properties.edit(self.policy, iface, show_versions = True)),
+		for label, cb in [(_('Show Feeds'), lambda: properties.edit(self.policy, iface, self.compile)),
+				  (_('Show Versions'), lambda: properties.edit(self.policy, iface, self.compile, show_versions = True)),
 				  (_('Report a Bug...'), lambda: bugs.report_bug(self.policy, iface))]:
 			item = gtk.MenuItem(label)
 			if cb:
@@ -394,24 +406,36 @@ class InterfaceBrowser:
 			item.set_submenu(compile_menu)
 
 			item = gtk.MenuItem(_('Automatic'))
-			item.connect('activate', lambda item: compile.compile(self.policy, iface, autocompile = True))
+			item.connect('activate', lambda item: self.compile(iface, autocompile = True))
 			item.show()
 			compile_menu.append(item)
 
 			item = gtk.MenuItem(_('Manual...'))
-			item.connect('activate', lambda item: compile.compile(self.policy, iface, autocompile = False))
+			item.connect('activate', lambda item: self.compile(iface, autocompile = False))
 			item.show()
 			compile_menu.append(item)
 		else:
 			item.set_sensitive(False)
 
 		menu.popup(None, None, None, bev.button, bev.time)
-	
+
+	def compile(self, interface, autocompile = True):
+		import compile
+		def on_success():
+			# A new local feed may have been registered, so reload it from the disk cache
+			info(_("0compile command completed successfully. Reloading interface details."))
+			reader.update_from_cache(interface)
+			for feed in interface.extra_feeds:
+				 self.policy.config.iface_cache.get_feed(feed.uri, force = True)
+			import main
+			main.recalculate()
+		compile.compile(on_success, interface.uri, autocompile = autocompile)
+
 	def set_original_implementations(self):
 		assert self.original_implementation is None
 		self.original_implementation = self.policy.implementation.copy()
 
-	def update_download_status(self):
+	def update_download_status(self, only_update_visible = False):
 		"""Called at regular intervals while there are downloads in progress,
 		and once at the end. Also called when things are added to the store.
 		Update the TreeView with the interfaces."""
@@ -424,20 +448,25 @@ class InterfaceBrowser:
 				if dl.hint not in hints:
 					hints[dl.hint] = []
 				hints[dl.hint].append(dl)
-			
+
 		selections = self.policy.solver.selections
 
-		def walk(it):
-			while it:
-				yield self.model[it]
-				for x in walk(self.model.iter_children(it)): yield x
-				it = self.model.iter_next(it)
+		# Only update currently visible rows
+		if only_update_visible and self.tree_view.get_visible_range() != None:
+			firstVisiblePath, lastVisiblePath = self.tree_view.get_visible_range()
+			firstVisibleIter = self.model.get_iter(firstVisiblePath)
+		else:
+			# (or should we just wait until the TreeView has settled enough to tell
+			# us what is visible?)
+			firstVisibleIter = self.model.get_iter_root()
+			lastVisiblePath = None
 
-		for row in walk(self.model.get_iter_root()):
+		for it in walk(self.model, firstVisibleIter):
+			row = self.model[it]
 			iface = row[InterfaceBrowser.INTERFACE]
-			
+
 			# Is this interface the download's hint?
-			downloads = hints.get(iface, [])	# The interface itself	
+			downloads = hints.get(iface, [])	# The interface itself
 		     	downloads += hints.get(iface.uri, [])	# The main feed
 			for feed in self.policy.usable_feeds(iface):
 				downloads += hints.get(feed.uri, []) # Other feeds
@@ -466,3 +495,16 @@ class InterfaceBrowser:
 			else:
 				row[InterfaceBrowser.DOWNLOAD_SIZE] = utils.get_fetch_info(self.policy, impl)
 				row[InterfaceBrowser.SUMMARY] = iface.summary
+
+			if self.model.get_path(it) == lastVisiblePath:
+				break
+
+	def highlight_problems(self):
+		"""Called when the solve finishes. Highlight any missing implementations."""
+		for it in walk(self.model, self.model.get_iter_root()):
+			row = self.model[it]
+			iface = row[InterfaceBrowser.INTERFACE]
+			sel = self.policy.solver.selections.selections.get(iface.uri, None)
+
+			if sel is None and row[InterfaceBrowser.PROBLEM]:
+				row[InterfaceBrowser.BACKGROUND] = '#f88'
