@@ -5,9 +5,10 @@ import os, sys
 
 from optparse import OptionParser
 
-from zeroinstall.injector import model, arch
+from zeroinstall import _
+from zeroinstall.injector import requirements
 from zeroinstall.injector.policy import Policy
-from zeroinstall.injector.iface_cache import iface_cache
+from zeroinstall.injector.config import load_config
 from zeroinstall.support import tasks
 
 _recalculate = tasks.Blocker('recalculate')
@@ -23,7 +24,6 @@ def run_gui(args):
 	parser = OptionParser(usage=_("usage: %prog [options] interface"))
 	parser.add_option("", "--before", help=_("choose a version before this"), metavar='VERSION')
 	parser.add_option("", "--cpu", help=_("target CPU type"), metavar='CPU')
-	parser.add_option("-c", "--cache", help=_("show the cache"), action='store_true')
 	parser.add_option("", "--command", help=_("command to select"), metavar='COMMAND')
 	parser.add_option("-d", "--download-only", help=_("fetch but don't run"), action='store_true')
 	parser.add_option("", "--message", help=_("message to display when interacting with user"))
@@ -49,18 +49,6 @@ def run_gui(args):
 		else:
 			logger.setLevel(logging.DEBUG)
 
-	if options.cache:
-		# Must fork before importing gtk, or ATK dies
-		if os.fork():
-			# We exit, so our parent can call waitpid and unblock.
-			sys.exit(0)
-		# The grandchild continues...
-
-	if options.with_store:
-		from zeroinstall import zerostore
-		for x in options.with_store:
-			iface_cache.stores.stores.append(zerostore.Store(os.path.abspath(x)))
-
 	import gui
 
 	if options.version:
@@ -78,31 +66,18 @@ def run_gui(args):
 		print >>sys.stderr, "Failed to connect to display. Aborting."
 		sys.exit(1)
 
-	if not hasattr(gtk, 'combo_box_new_text'):
-		import combo_compat
-
-	if options.cache:
-		import cache
-		cache_explorer = cache.CacheExplorer()
-		cache_explorer.show()
-		cache_explorer.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-		gtk.gdk.flush()
-		cache_explorer.populate_model()
-		cache_explorer.window.set_cursor(None)
-		gtk.main()
-		sys.exit(0)
-
 	handler = gui.GUIHandler()
 
+	config = load_config(handler)
+
+	if options.with_store:
+		from zeroinstall import zerostore
+		for x in options.with_store:
+			config.stores.stores.append(zerostore.Store(os.path.abspath(x)))
+
 	if len(args) < 1:
 		import preferences
-		# Once we separate configuration from Policy, this hack can go away
-		class DummyPolicy(Policy):
-			def recalculate(fetch_stale_interfaces = True):
-				pass
-			def solve_with_downloads(force = False):
-				pass
-		box = preferences.show_preferences(DummyPolicy('http://localhost/dummy', handler))
+		box = preferences.show_preferences(config)
 		box.connect('destroy', gtk.main_quit)
 		gtk.main()
 		sys.exit(0)
@@ -115,17 +90,13 @@ def run_gui(args):
 
 	import mainwindow, dialog
 
-	restrictions = []
-	if options.before or options.not_before:
-		restrictions.append(model.VersionRangeRestriction(model.parse_version(options.before),
-								  model.parse_version(options.not_before)))
+	r = requirements.Requirements(interface_uri)
+	r.parse_options(options)
 
 	widgets = dialog.Template('main')
 
-	policy = Policy(interface_uri, handler, src = bool(options.source), command = options.command)
-	policy.target_arch = arch.get_architecture(options.os, options.cpu)
-	root_iface = iface_cache.get_interface(interface_uri)
-	policy.solver.extra_restrictions[root_iface] = restrictions
+	policy = Policy(config = config, requirements = r)
+	root_iface = config.iface_cache.get_interface(interface_uri)
 	policy.solver.record_details = True
 
 	window = mainwindow.MainWindow(policy, widgets, download_only = bool(options.download_only), select_only = bool(options.select_only))
@@ -134,7 +105,7 @@ def run_gui(args):
 	if options.message:
 		window.set_message(options.message)
 
-	root = iface_cache.get_interface(policy.root)
+	root = config.iface_cache.get_interface(policy.root)
 	window.browser.set_root(root)
 
 	window.window.connect('destroy', lambda w: handler.abort_all_downloads())
@@ -156,8 +127,9 @@ def run_gui(args):
 			yield solved
 			try:
 				window.refresh_button.set_sensitive(True)
+				window.browser.highlight_problems()
 				tasks.check(solved)
-			except Exception, ex:
+			except Exception as ex:
 				window.report_exception(ex)
 
 			if window.systray_icon and window.systray_icon.get_visible() and \
@@ -176,4 +148,4 @@ def run_gui(args):
 			if refresh_clicked.happened:
 				force_refresh = True
 
-	handler.wait_for_blocker(main())
+	tasks.wait_for_blocker(main())
