@@ -5,6 +5,7 @@ import gtk
 import sys
 from logging import info
 
+from zeroinstall import _, translation
 from zeroinstall import SafeException
 from zeroinstall.support import tasks, pretty_size
 from zeroinstall.injector import download, iface_cache
@@ -13,6 +14,8 @@ import dialog
 from zeroinstall.gtkui import gtkutils
 from zeroinstall.gtkui import help_box
 
+ngettext = translation.ngettext
+
 SHOW_PREFERENCES = 0
 
 class MainWindow:
@@ -30,7 +33,11 @@ class MainWindow:
 		self.policy = policy
 		self.select_only = select_only
 
-		policy.watchers.append(lambda: self.window.set_response_sensitive(gtk.RESPONSE_OK, policy.solver.ready))
+		def update_ok_state():
+			self.window.set_response_sensitive(gtk.RESPONSE_OK, policy.solver.ready)
+			if policy.solver.ready and self.window.get_focus() is None:
+				run_button.grab_focus()
+		policy.watchers.append(update_ok_state)
 
 		self.window = widgets.get_widget('main')
 		self.window.set_default_size(gtk.gdk.screen_width() * 2 / 5, 300)
@@ -61,8 +68,7 @@ class MainWindow:
 		run_button.set_flags(gtk.CAN_DEFAULT)
 		self.run_button = run_button
 
-		self.window.set_default_response(gtk.RESPONSE_OK)
-		self.window.default_widget.grab_focus()
+		run_button.grab_focus()
 
 		def response(dialog, resp):
 			if resp in (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT):
@@ -78,7 +84,7 @@ class MainWindow:
 				gui_help.display()
 			elif resp == SHOW_PREFERENCES:
 				import preferences
-				preferences.show_preferences(policy)
+				preferences.show_preferences(policy.config, notify_cb = lambda: policy.solve_with_downloads())
 		self.window.connect('response', response)
 		self.window.realize()	# Make busy pointer work, even with --systray
 
@@ -115,7 +121,7 @@ class MainWindow:
 				dialog.alert(self.window, _('Not all downloads succeeded; cannot run program.\n\nFailed to get:') + '\n- ' + missing)
 			else:
 				from zeroinstall.injector import selections
-				sels = selections.Selections(self.policy)
+				sels = self.policy.solver.selections
 				doc = sels.toDOM()
 				reply = doc.toxml('utf-8')
 				sys.stdout.write(('Length:%8x\n' % len(reply)) + reply)
@@ -123,19 +129,19 @@ class MainWindow:
 				sys.exit(0)			# Success
 		except SystemExit:
 			raise
-		except download.DownloadAborted, ex:
+		except download.DownloadAborted as ex:
 			run_button.set_active(False)
 			# Don't bother reporting this to the user
-		except Exception, ex:
+		except Exception as ex:
 			run_button.set_active(False)
 			self.report_exception(ex)
 
-	def update_download_status(self):
+	def update_download_status(self, only_update_visible = False):
 		"""Called at regular intervals while there are downloads in progress,
 		and once at the end. Update the display."""
 		monitored_downloads = self.policy.handler.monitored_downloads
 
-		self.browser.update_download_status()
+		self.browser.update_download_status(only_update_visible)
 
 		if not monitored_downloads:
 			self.progress_area.hide()
@@ -180,8 +186,8 @@ class MainWindow:
 
 	def use_systray_icon(self):
 		try:
-			self.systray_icon = gtk.status_icon_new_from_icon_name("zeroinstall-zero2desktop")
-		except Exception, ex:
+			self.systray_icon = gtk.status_icon_new_from_icon_name("zeroinstall")
+		except Exception as ex:
 			info(_("No system tray support: %s"), ex)
 		else:
 			root_iface = iface_cache.iface_cache.get_interface(self.policy.root)
