@@ -1,6 +1,7 @@
 """
-The Python implementation of the Zero Install injector is divided into four sub-packages:
+The Python implementation of the Zero Install injector is divided into five sub-packages:
 
+ - L{zeroinstall.cmd} contains one module for each 0install sub-command (providing the shell-scripting interface)
  - L{zeroinstall.injector} contains most of the interesting stuff for managing feeds, keys and downloads and for selecting versions
  - L{zeroinstall.zerostore} contains low-level code for handling the implementation cache (where unpacked packages are stored)
  - L{zeroinstall.gtkui} contains code for making GTK user-interfaces
@@ -12,7 +13,7 @@ The Python implementation of the Zero Install injector is divided into four sub-
 @var _: a function for translating strings using the zero-install domain (for use internally by Zero Install)
 """
 
-version = '0.51'
+version = '1.2'
 
 import gettext
 from os.path import dirname, join
@@ -32,7 +33,7 @@ class SafeException(Exception):
 	The command-line interface's C{--verbose} option will display the full stack trace."""
 
 class NeedDownload(SafeException):
-	"""Thrown by L{injector.autopolicy.AutoPolicy} if we tried to start a download
-	and downloading is disabled."""
+	"""Thrown if we tried to start a download and downloading is
+	disabled."""
 	def __init__(self, url):
 		Exception.__init__(self, _("Would download '%s'") % url)
