@@ -8,7 +8,7 @@ Convenience routines for performing common operations.
 
 import os, sys, logging
 from zeroinstall import support
-from zeroinstall import _
+from zeroinstall.support import tasks
 
 def get_selections_gui(iface_uri, gui_args, test_callback = None):
 	"""Run the GUI to choose and download a set of implementations.
@@ -46,7 +46,7 @@ def get_selections_gui(iface_uri, gui_args, test_callback = None):
 					os.dup2(gui.fileno(), 0)
 					if iface_uri is not None:
 						gui_args = gui_args + ['--', iface_uri]
-					os.execvp(gui_exe, [gui_exe] + gui_args)
+					os.execvp(sys.executable, [sys.executable, gui_exe] + gui_args)
 				except:
 					import traceback
 					traceback.print_exc(file = sys.stderr)
@@ -99,7 +99,7 @@ def get_selections_gui(iface_uri, gui_args, test_callback = None):
 	
 	return sels
 
-def ensure_cached(uri, command = 'run'):
+def ensure_cached(uri, command = 'run', config = None):
 	"""Ensure that an implementation of uri is cached.
 	If not, it downloads one. It uses the GUI if a display is
 	available, or the console otherwise.
@@ -108,9 +108,12 @@ def ensure_cached(uri, command = 'run'):
 	@return: the selected implementations, or None if the user cancelled
 	@rtype: L{zeroinstall.injector.selections.Selections}
 	"""
-	from zeroinstall.injector import autopolicy, selections
+	from zeroinstall.injector import policy, selections
 
-	p = autopolicy.AutoPolicy(uri, download_only = True, command = command)
+	if config is None:
+		from zeroinstall.injector.config import load_config
+		config = load_config()
+	p = policy.Policy(uri, command = command, config = config)
 	p.freshness = 0		# Don't check for updates
 
 	if p.need_download() or not p.ready:
@@ -118,6 +121,6 @@ def ensure_cached(uri, command = 'run'):
 			return get_selections_gui(uri, ['--command', command])
 		else:
 			done = p.solve_and_download_impls()
-			p.handler.wait_for_blocker(done)
+			tasks.wait_for_blocker(done)
 
-	return selections.Selections(p)
+	return p.solver.selections
