@@ -7,10 +7,10 @@ Integration with native distribution package managers.
 # See the README file for details, or visit http://0install.net.
 
 from zeroinstall import _
-import os, platform, re, glob, subprocess, sys
+import os, platform, re, subprocess, sys
 from logging import warn, info
 from zeroinstall.injector import namespaces, model, arch
-from zeroinstall.support import basedir, tasks
+from zeroinstall.support import basedir
 
 _dotted_ints = '[0-9]+(?:\.[0-9]+)*'
 
@@ -35,7 +35,7 @@ class Cache(object):
 		self.cached_for = {}		# Attributes of source when cache was created
 		try:
 			self._load_cache()
-		except Exception, ex:
+		except Exception as ex:
 			info(_("Failed to load cache (%s). Flushing..."), ex)
 			self.flush()
 
@@ -45,7 +45,7 @@ class Cache(object):
 			info = os.stat(self.source)
 			mtime = int(info.st_mtime)
 			size = info.st_size
-		except Exception, ex:
+		except Exception as ex:
 			warn("Failed to stat %s: %s", self.source, ex)
 			mtime = size = 0
 		self.cache = {}
@@ -65,8 +65,6 @@ class Cache(object):
 		self.cache = cache = {}
 		stream = file(os.path.join(self.cache_dir, self.cache_leaf))
 		try:
-			meta = {}
-			cached_format = False
 			for line in stream:
 				line = line.strip()
 				if not line:
@@ -96,7 +94,7 @@ class Cache(object):
 	def get(self, key):
 		try:
 			self._check_valid()
-		except Exception, ex:
+		except Exception as ex:
 			info(_("Cache needs to be refreshed: %s"), ex)
 			self.flush()
 			return None
@@ -112,7 +110,7 @@ class Cache(object):
 				stream.write('%s=%s\n' % (key, value))
 			finally:
 				stream.close()
-		except Exception, ex:
+		except Exception as ex:
 			warn("Failed to write to cache %s: %s=%s: %s", cache_path, key, value, ex)
 
 def try_cleanup_distro_version(version):
@@ -203,6 +201,21 @@ class Distribution(object):
 				return impl
 
 			self.get_package_info(package, factory)
+
+		if master_feed.url == 'http://repo.roscidus.com/python/python' and all(not impl.installed for impl in feed.implementations.values()):
+			# Hack: we can support Python on platforms with unsupported package managers
+			# by adding the implementation of Python running us now to the list.
+			python_version = '.'.join([str(v) for v in sys.version_info if isinstance(v, int)])
+			impl_id = 'package:host:python:' + python_version
+			assert impl_id not in feed.implementations
+			impl = model.DistributionImplementation(feed, impl_id, self)
+			impl.installed = True
+			impl.version = model.parse_version(python_version)
+			impl.main = sys.executable
+			impl.upstream_stability = model.packaged
+			impl.machine = host_machine	# (hopefully)
+			feed.implementations[impl_id] = impl
+
 		return feed
 
 	def fetch_candidates(self, master_feed):
@@ -240,12 +253,12 @@ class CachedDistribution(Distribution):
 
 		try:
 			self._load_cache()
-		except Exception, ex:
+		except Exception as ex:
 			info(_("Failed to load distribution database cache (%s). Regenerating..."), ex)
 			try:
 				self.generate_cache()
 				self._load_cache()
-			except Exception, ex:
+			except Exception as ex:
 				warn(_("Failed to regenerate distribution database cache: %s"), ex)
 
 	def _load_cache(self):
@@ -425,7 +438,7 @@ class DebianDistribution(Distribution):
 				else:
 					cached = None
 				child.wait()
-			except Exception, ex:
+			except Exception as ex:
 				warn("'apt-cache show %s' failed: %s", package, ex)
 				cached = None
 			# (multi-arch support? can there be multiple candidates?)
