@@ -16,9 +16,6 @@ from zeroinstall.injector.iface_cache import PendingFeed, ReplayAttack
 from zeroinstall.injector.handler import NoTrustedKeys
 from zeroinstall.injector import download
 
-DEFAULT_FEED_MIRROR = "http://roscidus.com/0mirror"
-DEFAULT_KEY_LOOKUP_SERVER = 'https://keylookup.appspot.com'
-
 def _escape_slashes(path):
 	return path.replace('/', '%23')
 
@@ -41,14 +38,14 @@ class KeyInfoFetcher:
 
 	Example:
 
-	>>> kf = KeyInfoFetcher('https://server', fingerprint)
+	>>> kf = KeyInfoFetcher(handler, 'https://server', fingerprint)
 	>>> while True:
 		print kf.info
 		if kf.blocker is None: break
 		print kf.status
 		yield kf.blocker
 	"""
-	def __init__(self, server, fingerprint):
+	def __init__(self, handler, server, fingerprint):
 		self.fingerprint = fingerprint
 		self.info = []
 		self.blocker = None
@@ -57,8 +54,7 @@ class KeyInfoFetcher:
 
 		self.status = _('Fetching key information from %s...') % server
 
-		dl = download.Download(server + '/key/' + fingerprint)
-		dl.start()
+		dl = handler.get_download(server + '/key/' + fingerprint)
 
 		from xml.dom import minidom
 
@@ -74,7 +70,7 @@ class KeyInfoFetcher:
 				if doc.documentElement.localName != 'key-lookup':
 					raise SafeException(_('Expected <key-lookup>, not <%s>') % doc.documentElement.localName)
 				self.info += doc.documentElement.childNodes
-			except Exception, ex:
+			except Exception as ex:
 				doc = minidom.parseString('<item vote="bad"/>')
 				root = doc.documentElement
 				root.appendChild(doc.createTextNode(_('Error getting key information: %s') % ex))
@@ -84,23 +80,22 @@ class KeyInfoFetcher:
 
 class Fetcher(object):
 	"""Downloads and stores various things.
-	@ivar handler: handler to use for user-interaction
-	@type handler: L{handler.Handler}
+	@ivar config: used to get handler, iface_cache and stores
+	@type config: L{config.Config}
 	@ivar key_info: caches information about GPG keys
 	@type key_info: {str: L{KeyInfoFetcher}}
-	@ivar key_info_server: the base URL of a key information server
-	@type key_info_server: str
-	@ivar feed_mirror: the base URL of a mirror site for keys and feeds
-	@type feed_mirror: str | None
 	"""
-	__slots__ = ['handler', 'feed_mirror', 'key_info_server', 'key_info']
+	__slots__ = ['config', 'key_info']
 
-	def __init__(self, handler):
-		self.handler = handler
-		self.feed_mirror = DEFAULT_FEED_MIRROR
-		self.key_info_server = DEFAULT_KEY_LOOKUP_SERVER
+	def __init__(self, config):
+		assert config.handler, "API change!"
+		self.config = config
 		self.key_info = {}
 
+	@property
+	def handler(self):
+		return self.config.handler
+
 	@tasks.async
 	def cook(self, required_digest, recipe, stores, force = False, impl_hint = None):
 		"""Follow a Recipe.
@@ -109,7 +104,6 @@ class Fetcher(object):
 		# Maybe we're taking this metaphor too far?
 
 		# Start downloading all the ingredients.
-		downloads = {}	# Downloads that are not yet successful
 		streams = {}	# Streams collected from successful downloads
 
 		# Start a download for each ingredient
@@ -135,7 +129,10 @@ class Fetcher(object):
 			for step in recipe.steps:
 				stream = streams[step]
 				stream.seek(0)
-				unpack.unpack_archive_over(step.url, stream, tmpdir, step.extract)
+				unpack.unpack_archive_over(step.url, stream, tmpdir,
+						extract = step.extract,
+						type = step.type,
+						start_offset = step.start_offset or 0)
 			# Check that the result is correct and store it in the cache
 			store.check_manifest_and_rename(required_digest, tmpdir)
 			tmpdir = None
@@ -147,45 +144,46 @@ class Fetcher(object):
 
 	def get_feed_mirror(self, url):
 		"""Return the URL of a mirror for this feed."""
-		if self.feed_mirror is None:
+		if self.config.feed_mirror is None:
 			return None
 		import urlparse
 		if urlparse.urlparse(url).hostname == 'localhost':
 			return None
-		return '%s/%s/latest.xml' % (self.feed_mirror, _get_feed_dir(url))
+		return '%s/%s/latest.xml' % (self.config.feed_mirror, _get_feed_dir(url))
 
 	@tasks.async
-	def get_packagekit_feed(self, iface_cache, feed_url):
+	def get_packagekit_feed(self, feed_url):
 		"""Send a query to PackageKit (if available) for information about this package.
 		On success, the result is added to iface_cache.
 		"""
 		assert feed_url.startswith('distribution:'), feed_url
-		master_feed = iface_cache.get_feed(feed_url.split(':', 1)[1])
+		master_feed = self.config.iface_cache.get_feed(feed_url.split(':', 1)[1])
 		if master_feed:
-			fetch = iface_cache.distro.fetch_candidates(master_feed)
+			fetch = self.config.iface_cache.distro.fetch_candidates(master_feed)
 			if fetch:
 				yield fetch
 				tasks.check(fetch)
 
 			# Force feed to be regenerated with the new information
-			iface_cache.get_feed(feed_url, force = True)
+			self.config.iface_cache.get_feed(feed_url, force = True)
 
-	def download_and_import_feed(self, feed_url, iface_cache, force = False):
+	def download_and_import_feed(self, feed_url, iface_cache = None, force = False):
 		"""Download the feed, download any required keys, confirm trust if needed and import.
 		@param feed_url: the feed to be downloaded
 		@type feed_url: str
-		@param iface_cache: cache in which to store the feed
-		@type iface_cache: L{iface_cache.IfaceCache}
+		@param iface_cache: (deprecated)
 		@param force: whether to abort and restart an existing download"""
-		from download import DownloadAborted
+		from .download import DownloadAborted
+
+		assert iface_cache is None or iface_cache is self.config.iface_cache
 		
 		debug(_("download_and_import_feed %(url)s (force = %(force)d)"), {'url': feed_url, 'force': force})
 		assert not os.path.isabs(feed_url)
 
 		if feed_url.startswith('distribution:'):
-			return self.get_packagekit_feed(iface_cache, feed_url)
+			return self.get_packagekit_feed(feed_url)
 
-		primary = self._download_and_import_feed(feed_url, iface_cache, force, use_mirror = False)
+		primary = self._download_and_import_feed(feed_url, force, use_mirror = False)
 
 		@tasks.named_async("monitor feed downloads for " + feed_url)
 		def wait_for_downloads(primary):
@@ -202,20 +200,20 @@ class Fetcher(object):
 				# OK, maybe it's just being slow...
 				info("Feed download from %s is taking a long time.", feed_url)
 				primary_ex = None
-			except NoTrustedKeys, ex:
+			except NoTrustedKeys as ex:
 				raise			# Don't bother trying the mirror if we have a trust problem
-			except ReplayAttack, ex:
+			except ReplayAttack as ex:
 				raise			# Don't bother trying the mirror if we have a replay attack
-			except DownloadAborted, ex:
+			except DownloadAborted as ex:
 				raise			# Don't bother trying the mirror if the user cancelled
-			except SafeException, ex:
+			except SafeException as ex:
 				# Primary failed
 				primary = None
 				primary_ex = ex
 				warn(_("Feed download from %(url)s failed: %(exception)s"), {'url': feed_url, 'exception': ex})
 
 			# Start downloading from mirror...
-			mirror = self._download_and_import_feed(feed_url, iface_cache, force, use_mirror = True)
+			mirror = self._download_and_import_feed(feed_url, force, use_mirror = True)
 
 			# Wait until both mirror and primary tasks are complete...
 			while True:
@@ -233,7 +231,7 @@ class Fetcher(object):
 							if mirror:
 								info(_("Primary feed download succeeded; aborting mirror download for %s") % feed_url)
 								mirror.dl.abort()
-					except SafeException, ex:
+					except SafeException as ex:
 						primary = None
 						primary_ex = ex
 						info(_("Feed download from %(url)s failed; still trying mirror: %(exception)s"), {'url': feed_url, 'exception': ex})
@@ -247,11 +245,11 @@ class Fetcher(object):
 								# We already warned; no need to raise an exception too,
 								# as the mirror download succeeded.
 								primary_ex = None
-					except ReplayAttack, ex:
+					except ReplayAttack as ex:
 						info(_("Version from mirror is older than cached version; ignoring it: %s"), ex)
 						mirror = None
 						primary_ex = None
-					except SafeException, ex:
+					except SafeException as ex:
 						info(_("Mirror download failed: %s"), ex)
 						mirror = None
 
@@ -260,13 +258,13 @@ class Fetcher(object):
 
 		return wait_for_downloads(primary)
 
-	def _download_and_import_feed(self, feed_url, iface_cache, force, use_mirror):
+	def _download_and_import_feed(self, feed_url, force, use_mirror):
 		"""Download and import a feed.
 		@param use_mirror: False to use primary location; True to use mirror."""
 		if use_mirror:
 			url = self.get_feed_mirror(feed_url)
 			if url is None: return None
-			warn(_("Trying mirror server for feed %s") % feed_url)
+			info(_("Trying mirror server for feed %s") % feed_url)
 		else:
 			url = feed_url
 
@@ -282,7 +280,7 @@ class Fetcher(object):
 
 			if use_mirror:
 				# If we got the feed from a mirror, get the key from there too
-				key_mirror = self.feed_mirror + '/keys/'
+				key_mirror = self.config.feed_mirror + '/keys/'
 			else:
 				key_mirror = None
 
@@ -290,12 +288,12 @@ class Fetcher(object):
 			yield keys_downloaded.finished
 			tasks.check(keys_downloaded.finished)
 
-			if not iface_cache.update_feed_if_trusted(pending.url, pending.sigs, pending.new_xml):
-				blocker = self.handler.confirm_keys(pending, self.fetch_key_info)
+			if not self.config.iface_cache.update_feed_if_trusted(pending.url, pending.sigs, pending.new_xml):
+				blocker = self.config.trust_mgr.confirm_keys(pending)
 				if blocker:
 					yield blocker
 					tasks.check(blocker)
-				if not iface_cache.update_feed_if_trusted(pending.url, pending.sigs, pending.new_xml):
+				if not self.config.iface_cache.update_feed_if_trusted(pending.url, pending.sigs, pending.new_xml):
 					raise NoTrustedKeys(_("No signing keys trusted; not importing"))
 
 		task = fetch_feed()
@@ -306,8 +304,9 @@ class Fetcher(object):
 		try:
 			return self.key_info[fingerprint]
 		except KeyError:
-			self.key_info[fingerprint] = info = KeyInfoFetcher(self.key_info_server, fingerprint)
-			return info
+			self.key_info[fingerprint] = key_info = KeyInfoFetcher(self.handler,
+									self.config.key_info_server, fingerprint)
+			return key_info
 
 	def download_impl(self, impl, retrieval_method, stores, force = False):
 		"""Download an implementation.
@@ -362,7 +361,6 @@ class Fetcher(object):
 
 	def _add_to_cache(self, required_digest, stores, retrieval_method, stream):
 		assert isinstance(retrieval_method, DownloadSource)
-		url = retrieval_method.url
 		stores.add_archive_to_cache(required_digest, stream, retrieval_method.url, retrieval_method.extract,
 						 type = retrieval_method.type, start_offset = retrieval_method.start_offset or 0)
 
@@ -385,12 +383,19 @@ class Fetcher(object):
 		dl.expected_size = download_source.size + (download_source.start_offset or 0)
 		return (dl.downloaded, dl.tempfile)
 
-	def download_icon(self, interface, force = False, modification_time = None):
+	def download_icon(self, interface, force = False):
 		"""Download an icon for this interface and add it to the
-		icon cache. If the interface has no icon or we are offline, do nothing.
+		icon cache. If the interface has no icon do nothing.
 		@return: the task doing the import, or None
 		@rtype: L{tasks.Task}"""
-		debug(_("download_icon %(interface)s (force = %(force)d)"), {'interface': interface, 'force': force})
+		debug("download_icon %(interface)s (force = %(force)d)", {'interface': interface, 'force': force})
+
+		modification_time = None
+		existing_icon = self.config.iface_cache.get_icon_path(interface)
+		if existing_icon:
+			file_mtime = os.stat(existing_icon).st_mtime
+			from email.utils import formatdate
+			modification_time = formatdate(timeval = file_mtime, localtime = False, usegmt = True)
 
 		# Find a suitable icon to download
 		for icon in interface.get_metadata(XMLNS_IFACE, 'icon'):
@@ -428,7 +433,7 @@ class Fetcher(object):
 				icons_cache = basedir.save_cache_path(config_site, 'interface_icons')
 				icon_file = file(os.path.join(icons_cache, escape(interface.uri)), 'w')
 				shutil.copyfileobj(stream, icon_file)
-			except Exception, ex:
+			except Exception as ex:
 				self.handler.report_error(ex)
 
 		return download_and_add_icon()
