@@ -37,7 +37,10 @@ from zeroinstall.support import basedir
 from zeroinstall.injector import reader, model
 from zeroinstall.injector.namespaces import config_site, config_prog
 from zeroinstall.injector.model import Interface, escape, unescape
-from zeroinstall import zerostore, SafeException
+from zeroinstall import SafeException
+
+# If we started a check within this period, don't start another one:
+FAILED_CHECK_DELAY = 60 * 60	# 1 Hour
 
 def _pretty_time(t):
 	assert isinstance(t, (int, long)), t
@@ -144,7 +147,7 @@ class PendingFeed(object):
 	def recheck(self):
 		"""Set new_xml and sigs by reading signed_data.
 		You need to call this when previously-missing keys are added to the GPG keyring."""
-		import gpg
+		from . import gpg
 		try:
 			self.signed_data.seek(0)
 			stream, sigs = gpg.check_stream(self.signed_data)
@@ -180,7 +183,7 @@ class IfaceCache(object):
 	@see: L{iface_cache} - the singleton IfaceCache instance.
 	"""
 
-	__slots__ = ['_interfaces', 'stores', '_feeds', '_distro']
+	__slots__ = ['_interfaces', '_feeds', '_distro', '_config']
 
 	def __init__(self, distro = None):
 		"""@param distro: distribution used to fetch "distribution:" feeds (since 0.49)
@@ -189,18 +192,20 @@ class IfaceCache(object):
 		"""
 		self._interfaces = {}
 		self._feeds = {}
-
-		self.stores = zerostore.Stores()
-
 		self._distro = distro
 
+	@property
+	def stores(self):
+		from zeroinstall.injector import policy
+		return policy.get_deprecated_singleton_config().stores
+
 	@property
 	def distro(self):
 		if self._distro is None:
 			from zeroinstall.injector.distro import get_host_distribution
 			self._distro = get_host_distribution()
 		return self._distro
-	
+
 	def update_interface_if_trusted(self, interface, sigs, xml):
 		import warnings
 		warnings.warn("Use update_feed_if_trusted instead", DeprecationWarning, stacklevel = 2)
@@ -220,10 +225,10 @@ class IfaceCache(object):
 		@rtype: bool
 		@since: 0.48
 		"""
-		import trust
+		from . import trust
 		updated = self._oldest_trusted(sigs, trust.domain_from_url(feed_url))
 		if updated is None: return False	# None are trusted
-	
+
 		self.update_feed_from_network(feed_url, xml, updated)
 		return True
 
@@ -266,8 +271,8 @@ class IfaceCache(object):
 
 		feed = self.get_feed(feed_url)
 
-		import writer
-		feed.last_checked = long(time.time())
+		from . import writer
+		feed.last_checked = int(time.time())
 		writer.save_feed(feed)
 
 		info(_("Updated feed cache entry for %(interface)s (modified %(time)s)"),
@@ -325,10 +330,11 @@ class IfaceCache(object):
 
 		self.get_feed(feed_url, force = True)
 
-	def get_feed(self, url, force = False):
+	def get_feed(self, url, force = False, selections_ok = False):
 		"""Get a feed from the cache.
 		@param url: the URL of the feed
 		@param force: load the file from disk again
+		@param selections_ok: if url is a local selections file, return that instead
 		@return: the feed, or None if it isn't cached
 		@rtype: L{model.ZeroInstallFeed}"""
 		if not force:
@@ -342,7 +348,10 @@ class IfaceCache(object):
 				return None	# Can't happen?
 			feed = self.distro.get_feed(master_feed)
 		else:
-			feed = reader.load_feed_from_cache(url)
+			feed = reader.load_feed_from_cache(url, selections_ok = selections_ok)
+			if selections_ok and feed and not isinstance(feed, model.ZeroInstallFeed):
+				assert feed.selections is not None
+				return feed	# (it's actually a selections document)
 		if feed:
 			reader.update_user_feed_overrides(feed)
 		self._feeds[url] = feed
@@ -364,7 +373,7 @@ class IfaceCache(object):
 
 		debug(_("Initialising new interface object for %s"), uri)
 		self._interfaces[uri] = Interface(uri)
-		reader.update_from_cache(self._interfaces[uri])
+		reader.update_from_cache(self._interfaces[uri], iface_cache = self)
 		return self._interfaces[uri]
 
 	def list_all_interfaces(self):
@@ -376,10 +385,6 @@ class IfaceCache(object):
 			for leaf in os.listdir(d):
 				if not leaf.startswith('.'):
 					all.add(unescape(leaf))
-		for d in basedir.load_config_paths(config_site, config_prog, 'user_overrides'):
-			for leaf in os.listdir(d):
-				if not leaf.startswith('.'):
-					all.add(unescape(leaf))
 		return list(all)	# Why not just return the set?
 
 	def get_icon_path(self, iface):
@@ -398,7 +403,7 @@ class IfaceCache(object):
 		@return: a list of signatures, or None
 		@rtype: [L{gpg.Signature}] or None
 		@since: 0.25"""
-		import gpg
+		from . import gpg
 		if os.path.isabs(uri):
 			old_iface = uri
 		else:
@@ -407,18 +412,18 @@ class IfaceCache(object):
 				return None
 		try:
 			return gpg.check_stream(file(old_iface))[1]
-		except SafeException, ex:
+		except SafeException as ex:
 			debug(_("No signatures (old-style interface): %s") % ex)
 			return None
-	
+
 	def _get_signature_date(self, uri):
 		"""Read the date-stamp from the signature of the cached interface.
 		If the date-stamp is unavailable, returns None."""
-		import trust
+		from . import trust
 		sigs = self.get_cached_signatures(uri)
 		if sigs:
 			return self._oldest_trusted(sigs, trust.domain_from_url(uri))
-	
+
 	def _oldest_trusted(self, sigs, domain):
 		"""Return the date of the oldest trusted signature in the list, or None if there
 		are no trusted sigs in the list."""
@@ -436,7 +441,7 @@ class IfaceCache(object):
 			return
 		feeds_dir = basedir.save_cache_path(config_site, config_prog, 'last-check-attempt')
 		timestamp_path = os.path.join(feeds_dir, model._pretty_escape(url))
-		fd = os.open(timestamp_path, os.O_WRONLY | os.O_CREAT, 0644)
+		fd = os.open(timestamp_path, os.O_WRONLY | os.O_CREAT, 0o644)
 		os.close(fd)
 		os.utime(timestamp_path, None)	# In case file already exists
 
@@ -474,7 +479,7 @@ class IfaceCache(object):
 		for imp in iface.extra_feeds:
 			try:
 				results[imp.uri] = self.get_feed(imp.uri)
-			except SafeException, ex:
+			except SafeException as ex:
 				warn("Failed to load feed '%s: %s", imp.uri, ex)
 		if main_feed:
 			for imp in main_feed.feeds:
@@ -491,4 +496,64 @@ class IfaceCache(object):
 				impls += feed.implementations.values()
 		return impls
 
+	def get_feed_targets(self, feed):
+		"""Return a list of Interfaces for which feed can be a feed.
+		This is used by B{0install add-feed}.
+		@param feed: the feed
+		@type feed: L{model.ZeroInstallFeed} (or, deprecated, a URL)
+		@rtype: [model.Interface]
+		@raise SafeException: If there are no known feeds.
+		@since: 0.53"""
+
+		if not isinstance(feed, model.ZeroInstallFeed):
+			# (deprecated)
+			feed = self.get_feed(feed)
+			if feed is None:
+				raise SafeException("Feed is not cached and using deprecated API")
+
+		if not feed.feed_for:
+			raise SafeException(_("Missing <feed-for> element in '%s'; "
+					"it can't be used as a feed for any other interface.") % feed.url)
+		feed_targets = feed.feed_for
+		debug(_("Feed targets: %s"), feed_targets)
+		return [self.get_interface(uri) for uri in feed_targets]
+
+	def is_stale(self, feed, freshness_threshold):
+		"""Check whether feed needs updating, based on the configured L{freshness}.
+		None is considered to be stale.
+		If we already tried to update the feed within FAILED_CHECK_DELAY, returns false.
+		@return: True if feed should be updated
+		@since: 0.53"""
+		if feed is None:
+			return True
+		if os.path.isabs(feed.url):
+			return False		# Local feeds are never stale
+		if feed.last_modified is None:
+			return True		# Don't even have it yet
+		now = time.time()
+		staleness = now - (feed.last_checked or 0)
+		debug(_("Staleness for %(feed)s is %(staleness).2f hours"), {'feed': feed, 'staleness': staleness / 3600.0})
+
+		if freshness_threshold <= 0 or staleness < freshness_threshold:
+			return False		# Fresh enough for us
+
+		last_check_attempt = self.get_last_check_attempt(feed.url)
+		if last_check_attempt and last_check_attempt > now - FAILED_CHECK_DELAY:
+			debug(_("Stale, but tried to check recently (%s) so not rechecking now."), time.ctime(last_check_attempt))
+			return False
+
+		return True
+
+	def usable_feeds(self, iface, arch):
+		"""Generator for C{iface.feeds} that are valid for this architecture.
+		@rtype: generator
+		@see: L{arch}
+		@since: 0.53"""
+		for f in self.get_feed_imports(iface):
+			if f.os in arch.os_ranks and f.machine in arch.machine_ranks:
+				yield f
+			else:
+				debug(_("Skipping '%(feed)s'; unsupported architecture %(os)s-%(machine)s"),
+					{'feed': f, 'os': f.os, 'machine': f.machine})
+
 iface_cache = IfaceCache()
