@@ -21,7 +21,7 @@ from zeroinstall.injector.namespaces import XMLNS_IFACE
 from zeroinstall.injector import qdom
 
 # Element names for bindings in feed files
-binding_names = frozenset(['environment', 'overlay'])
+binding_names = frozenset(['environment', 'overlay', 'executable-in-path', 'executable-in-var'])
 
 network_offline = 'off-line'
 network_minimal = 'minimal'
@@ -55,10 +55,14 @@ class InvalidInterface(SafeException):
 
 		SafeException.__init__(self, message)
 
-	def __str__(self):
-		if self.feed_url:
-			return SafeException.__str__(self) + ' in ' + self.feed_url
-		return SafeException.__str__(self)
+	def __unicode__(self):
+		if hasattr(SafeException, '__unicode__'):
+			# Python >= 2.6
+			if self.feed_url:
+				return _('%s [%s]') % (SafeException.__unicode__(self), self.feed_url)
+			return SafeException.__unicode__(self)
+		else:
+			return unicode(SafeException.__str__(self))
 
 def _split_arch(arch):
 	"""Split an arch into an (os, machine) tuple. Either or both parts may be None."""
@@ -77,10 +81,17 @@ def _join_arch(osys, machine):
 	return "%s-%s" % (osys or '*', machine or '*')
 
 def _best_language_match(options):
-	(language, encoding) = locale.getlocale(locale.LC_ALL)
-	return (options.get(language, None) or
-		options.get(language.split('_', 1)[0], None) or
-		options.get(None, None))
+	(language, encoding) = locale.getlocale()
+
+	if language:
+		# xml:lang uses '-', while LANG uses '_'
+		language = language.replace('_', '-')
+	else:
+		language = 'en-US'
+
+	return (options.get(language, None) or			# Exact match (language+region)
+		options.get(language.split('-', 1)[0], None) or	# Matching language
+		options.get('en', None))			# English
 
 class Stability(object):
 	"""A stability rating. Each implementation has an upstream stability rating and,
@@ -115,10 +126,19 @@ def process_binding(e):
 		binding = EnvironmentBinding(e.getAttribute('name'),
 					     insert = e.getAttribute('insert'),
 					     default = e.getAttribute('default'),
-					     mode = mode)
+					     value = e.getAttribute('value'),
+					     mode = mode,
+					     separator = e.getAttribute('separator'))
 		if not binding.name: raise InvalidInterface(_("Missing 'name' in binding"))
-		if binding.insert is None: raise InvalidInterface(_("Missing 'insert' in binding"))
+		if binding.insert is None and binding.value is None:
+			raise InvalidInterface(_("Missing 'insert' or 'value' in binding"))
+		if binding.insert is not None and binding.value is not None:
+			raise InvalidInterface(_("Binding contains both 'insert' and 'value'"))
 		return binding
+	elif e.name == 'executable-in-path':
+		return ExecutableBinding(e, in_path = True)
+	elif e.name == 'executable-in-var':
+		return ExecutableBinding(e, in_path = False)
 	elif e.name == 'overlay':
 		return OverlayBinding(e.getAttribute('src'), e.getAttribute('mount-point'))
 	else:
@@ -224,23 +244,38 @@ class Binding(object):
 	"""Information about how the choice of a Dependency is made known
 	to the application being run."""
 
+	@property
+	def command(self):
+		""""Returns the name of the specific command needed by this binding, if any.
+		@since: 1.2"""
+		return None
+
 class EnvironmentBinding(Binding):
 	"""Indicate the chosen implementation using an environment variable."""
-	__slots__ = ['name', 'insert', 'default', 'mode']
+	__slots__ = ['name', 'insert', 'default', 'mode', 'value']
 
 	PREPEND = 'prepend'
 	APPEND = 'append'
 	REPLACE = 'replace'
 
-	def __init__(self, name, insert, default = None, mode = PREPEND):
-		"""mode argument added in version 0.28"""
+	def __init__(self, name, insert, default = None, mode = PREPEND, value=None, separator=None):
+		"""
+		mode argument added in version 0.28
+		value argument added in version 0.52
+		"""
 		self.name = name
 		self.insert = insert
 		self.default = default
 		self.mode = mode
+		self.value = value
+		if separator is None:
+			self.separator = os.pathsep
+		else:
+			self.separator = separator
+
 	
 	def __str__(self):
-		return _("<environ %(name)s %(mode)s %(insert)s>") % {'name': self.name,'mode':  self.mode, 'insert': self.insert}
+		return _("<environ %(name)s %(mode)s %(insert)s %(value)s>") % {'name': self.name,'mode': self.mode, 'insert': self.insert, 'value': self.value}
 
 	__repr__ = __str__
 	
@@ -249,7 +284,12 @@ class EnvironmentBinding(Binding):
 		@param path: the path to the selected implementation
 		@param old_value: the current value of the environment variable
 		@return: the new value for the environment variable"""
-		extra = os.path.join(path, self.insert)
+
+		if self.insert is not None:
+			extra = os.path.join(path, self.insert)
+		else:
+			assert self.value is not None
+			extra = self.value
 
 		if self.mode == EnvironmentBinding.REPLACE:
 			return extra
@@ -259,22 +299,56 @@ class EnvironmentBinding(Binding):
 		if old_value is None:
 			return extra
 		if self.mode == EnvironmentBinding.PREPEND:
-			return extra + os.pathsep + old_value
+			return extra + self.separator + old_value
 		else:
-			return old_value + os.pathsep + extra
+			return old_value + self.separator + extra
 
-	def _toxml(self, doc):
+	def _toxml(self, doc, prefixes):
 		"""Create a DOM element for this binding.
 		@param doc: document to use to create the element
 		@return: the new element
 		"""
 		env_elem = doc.createElementNS(XMLNS_IFACE, 'environment')
 		env_elem.setAttributeNS(None, 'name', self.name)
-		env_elem.setAttributeNS(None, 'insert', self.insert)
+		if self.mode is not None:
+			env_elem.setAttributeNS(None, 'mode', self.mode)
+		if self.insert is not None:
+			env_elem.setAttributeNS(None, 'insert', self.insert)
+		else:
+			env_elem.setAttributeNS(None, 'value', self.value)
 		if self.default:
 			env_elem.setAttributeNS(None, 'default', self.default)
+		if self.separator:
+			env_elem.setAttributeNS(None, 'separator', self.separator)
 		return env_elem
 
+class ExecutableBinding(Binding):
+	"""Make the chosen command available in $PATH.
+	@ivar in_path: True to add the named command to $PATH, False to store in named variable
+	@type in_path: bool
+	"""
+	__slots__ = ['qdom']
+
+	def __init__(self, qdom, in_path):
+		self.qdom = qdom
+		self.in_path = in_path
+
+	def __str__(self):
+		return str(self.qdom)
+
+	__repr__ = __str__
+
+	def _toxml(self, doc, prefixes):
+		return self.qdom.toDOM(doc, prefixes)
+
+	@property
+	def name(self):
+		return self.qdom.getAttribute('name')
+
+	@property
+	def command(self):
+		return self.qdom.getAttribute("command") or 'run'
+
 class OverlayBinding(Binding):
 	"""Make the chosen implementation available by overlaying it onto another part of the file-system.
 	This is to support legacy programs which use hard-coded paths."""
@@ -329,6 +403,9 @@ class Dependency(object):
 	"""
 	__slots__ = ['qdom']
 
+	Essential = "essential"
+	Recommended = "recommended"
+
 	def __init__(self, element):
 		assert isinstance(element, qdom.Element), type(element)	# Use InterfaceDependency instead!
 		self.qdom = element
@@ -337,6 +414,14 @@ class Dependency(object):
 	def metadata(self):
 		return self.qdom.attrs
 
+	@property
+	def importance(self):
+		return self.qdom.getAttribute("importance") or Dependency.Essential
+
+	def get_required_commands(self):
+		"""Return a list of command names needed by this dependency"""
+		return []
+
 class InterfaceDependency(Dependency):
 	"""A Dependency on a Zero Install interface.
 	@ivar interface: the interface required by this dependency
@@ -363,6 +448,24 @@ class InterfaceDependency(Dependency):
 	def __str__(self):
 		return _("<Dependency on %(interface)s; bindings: %(bindings)s%(restrictions)s>") % {'interface': self.interface, 'bindings': self.bindings, 'restrictions': self.restrictions}
 
+	def get_required_commands(self):
+		"""Return a list of command names needed by this dependency"""
+		if self.qdom.name == 'runner':
+			commands = [self.qdom.getAttribute('command') or 'run']
+		else:
+			commands = []
+		for b in self.bindings:
+			c = b.command
+			if c is not None:
+				commands.append(c)
+		return commands
+
+	@property
+	def command(self):
+		if self.qdom.name == 'runner':
+			return self.qdom.getAttribute('command') or 'run'
+		return None
+
 class RetrievalMethod(object):
 	"""A RetrievalMethod provides a way to fetch an implementation."""
 	__slots__ = []
@@ -453,6 +556,9 @@ class Command(object):
 		self.requires		# (sets _runner)
 		return self._runner
 
+	def __str__(self):
+		return str(self.qdom)
+
 class Implementation(object):
 	"""An Implementation is a package which implements an Interface.
 	@ivar download_sources: list of methods of getting this implementation
@@ -551,6 +657,14 @@ class Implementation(object):
 			self.commands["run"] = Command(qdom.Element(XMLNS_IFACE, 'command', {'path': path}), None)
 	main = property(_get_main, _set_main)
 
+	def is_available(self, stores):
+		"""Is this Implementation available locally?
+		(a local implementation, an installed distribution package, or a cached ZeroInstallImplementation)
+		@rtype: bool
+		@since: 0.53
+		"""
+		raise NotImplementedError("abstract")
+
 class DistributionImplementation(Implementation):
 	"""An implementation provided by the distribution. Information such as the version
 	comes from the package manager.
@@ -567,6 +681,9 @@ class DistributionImplementation(Implementation):
 	def requires_root_install(self):
 		return not self.installed
 
+	def is_available(self, stores):
+		return self.installed
+
 class ZeroInstallImplementation(Implementation):
 	"""An implementation where all the information comes from Zero Install.
 	@ivar digests: a list of "algorith=value" strings (since 0.45)
@@ -595,6 +712,14 @@ class ZeroInstallImplementation(Implementation):
 		self.os, self.machine = _split_arch(arch)
 	arch = property(lambda self: _join_arch(self.os, self.machine), set_arch)
 
+	def is_available(self, stores):
+		if self.local_path is not None:
+			return os.path.exists(self.local_path)
+		if self.digests:
+			path = stores.lookup_maybe(self.digests)
+			return path is not None
+		return False	# (0compile creates fake entries with no digests)
+
 class Interface(object):
 	"""An Interface represents some contract of behaviour.
 	@ivar uri: the URI for this interface.
@@ -664,7 +789,8 @@ class Interface(object):
 	def _main_feed(self):
 		#import warnings
 		#warnings.warn("use the feed instead", DeprecationWarning, 3)
-		from zeroinstall.injector.iface_cache import iface_cache
+		from zeroinstall.injector import policy
+		iface_cache = policy.get_deprecated_singleton_config().iface_cache
 		feed = iface_cache.get_feed(self.uri)
 		if feed is None:
 			return _dummy_feed
@@ -685,8 +811,8 @@ def _get_long(elem, attr_name):
 	val = elem.getAttribute(attr_name)
 	if val is not None:
 		try:
-			val = long(val)
-		except ValueError, ex:
+			val = int(val)
+		except ValueError:
 			raise SafeException(_("Invalid value for integer attribute '%(attribute_name)s': %(value)s") % {'attribute_name': attr_name, 'value': val})
 	return val
 
@@ -709,7 +835,7 @@ class ZeroInstallFeed(object):
 	@ivar metadata: extra elements we didn't understand
 	"""
 	# _main is deprecated
-	__slots__ = ['url', 'implementations', 'name', 'descriptions', 'summaries', '_package_implementations',
+	__slots__ = ['url', 'implementations', 'name', 'descriptions', 'first_description', 'summaries', 'first_summary', '_package_implementations',
 		     'last_checked', 'last_modified', 'feeds', 'feed_for', 'metadata']
 
 	def __init__(self, feed_element, local_path = None, distro = None):
@@ -720,7 +846,9 @@ class ZeroInstallFeed(object):
 		self.implementations = {}
 		self.name = None
 		self.summaries = {}	# { lang: str }
+		self.first_summary = None
 		self.descriptions = {}	# { lang: str }
+		self.first_description = None
 		self.last_modified = None
 		self.feeds = []
 		self.feed_for = set()
@@ -765,9 +893,13 @@ class ZeroInstallFeed(object):
 			if x.name == 'name':
 				self.name = x.content
 			elif x.name == 'description':
-				self.descriptions[x.attrs.get("http://www.w3.org/XML/1998/namespace lang", None)] = x.content
+				if self.first_description == None:
+					self.first_description = x.content
+				self.descriptions[x.attrs.get("http://www.w3.org/XML/1998/namespace lang", 'en')] = x.content
 			elif x.name == 'summary':
-				self.summaries[x.attrs.get("http://www.w3.org/XML/1998/namespace lang", None)] = x.content
+				if self.first_summary == None:
+					self.first_summary = x.content
+				self.summaries[x.attrs.get("http://www.w3.org/XML/1998/namespace lang", 'en')] = x.content
 			elif x.name == 'feed-for':
 				feed_iface = x.getAttribute('interface')
 				if not feed_iface:
@@ -782,7 +914,9 @@ class ZeroInstallFeed(object):
 				if not feed_src:
 					raise InvalidInterface(_('Missing "src" attribute in <feed>'))
 				if feed_src.startswith('http:') or feed_src.startswith('https:') or local_path:
-					self.feeds.append(Feed(feed_src, x.getAttribute('arch'), False, langs = x.getAttribute('langs')))
+					langs = x.getAttribute('langs')
+					if langs: langs = langs.replace('_', '-')
+					self.feeds.append(Feed(feed_src, x.getAttribute('arch'), False, langs = langs))
 				else:
 					raise InvalidInterface(_("Invalid feed URL '%s'") % feed_src)
 			else:
@@ -800,18 +934,23 @@ class ZeroInstallFeed(object):
 				if item.name not in ('group', 'implementation', 'package-implementation'):
 					continue
 
-				depends = base_depends[:]
-				bindings = base_bindings[:]
-				commands = base_commands.copy()
-
-				item_attrs = _merge_attrs(group_attrs, item)
-
-				# We've found a group or implementation. Scan for dependencies
-				# and bindings. Doing this here means that:
+				# We've found a group or implementation. Scan for dependencies,
+				# bindings and commands. Doing this here means that:
 				# - We can share the code for groups and implementations here.
 				# - The order doesn't matter, because these get processed first.
 				# A side-effect is that the document root cannot contain
 				# these.
+
+				depends = base_depends[:]
+				bindings = base_bindings[:]
+				commands = base_commands.copy()
+
+				for attr, command in [('main', 'run'),
+						      ('self-test', 'test')]:
+					value = item.attrs.get(attr, None)
+					if value is not None:
+						commands[command] = Command(qdom.Element(XMLNS_IFACE, 'command', {'name': command, 'path': value}), None)
+
 				for child in item.childNodes:
 					if child.uri != XMLNS_IFACE: continue
 					if child.name == 'requires':
@@ -825,15 +964,11 @@ class ZeroInstallFeed(object):
 					elif child.name in binding_names:
 						bindings.append(process_binding(child))
 
-				for attr, command in [('main', 'run'),
-						      ('self-test', 'test')]:
-					value = item.attrs.get(attr, None)
-					if value is not None:
-						commands[command] = Command(qdom.Element(XMLNS_IFACE, 'command', {'path': value}), None)
-
 				compile_command = item.attrs.get('http://zero-install.sourceforge.net/2006/namespaces/0compile command')
 				if compile_command is not None:
-					commands['compile'] = Command(qdom.Element(XMLNS_IFACE, 'command', {'shell-command': compile_command}), None)
+					commands['compile'] = Command(qdom.Element(XMLNS_IFACE, 'command', {'name': 'compile', 'shell-command': compile_command}), None)
+
+				item_attrs = _merge_attrs(group_attrs, item)
 
 				if item.name == 'group':
 					process_group(item, item_attrs, depends, bindings, commands)
@@ -878,15 +1013,14 @@ class ZeroInstallFeed(object):
 				raise InvalidInterface(_("Missing version attribute"))
 			impl.version = parse_version(version)
 
-			item_main = commands.get('run', None)
 			impl.commands = commands
 
 			impl.released = item_attrs.get('released', None)
-			impl.langs = item_attrs.get('langs', '')
+			impl.langs = item_attrs.get('langs', '').replace('_', '-')
 
 			size = item.getAttribute('size')
 			if size:
-				impl.size = long(size)
+				impl.size = int(size)
 			impl.arch = item_attrs.get('arch', None)
 			try:
 				stability = stability_levels[str(item_attrs['stability'])]
@@ -911,7 +1045,7 @@ class ZeroInstallFeed(object):
 					size = elem.getAttribute('size')
 					if not size:
 						raise InvalidInterface(_("Missing size attribute on <archive>"))
-					impl.add_download_source(url = url, size = long(size),
+					impl.add_download_source(url = url, size = int(size),
 							extract = elem.getAttribute('extract'),
 							start_offset = _get_long(elem, 'start-offset'),
 							type = elem.getAttribute('type'))
@@ -929,7 +1063,7 @@ class ZeroInstallFeed(object):
 							size = recipe_step.getAttribute('size')
 							if not size:
 								raise InvalidInterface(_("Missing size attribute on <archive>"))
-							recipe.steps.append(DownloadSource(None, url = url, size = long(size),
+							recipe.steps.append(DownloadSource(None, url = url, size = int(size),
 									extract = recipe_step.getAttribute('extract'),
 									start_offset = _get_long(recipe_step, 'start-offset'),
 									type = recipe_step.getAttribute('type')))
@@ -982,21 +1116,6 @@ class ZeroInstallFeed(object):
 	def __repr__(self):
 		return _("<Feed %s>") % self.url
 	
-	"""@deprecated"""
-	def _get_impl(self, id):
-		assert id not in self.implementations
-
-		if id.startswith('.') or id.startswith('/'):
-			id = os.path.abspath(os.path.join(self.url, id))
-			local_path = id
-			impl = ZeroInstallImplementation(self, id, local_path)
-		else:
-			impl = ZeroInstallImplementation(self, id, None)
-			impl.digests.append(id)
-
-		self.implementations[id] = impl
-		return impl
-	
 	def set_stability_policy(self, new):
 		assert new is None or isinstance(new, Stability)
 		self.stability_policy = new
@@ -1016,11 +1135,11 @@ class ZeroInstallFeed(object):
 
 	@property
 	def summary(self):
-		return _best_language_match(self.summaries)
+		return _best_language_match(self.summaries) or self.first_summary
 
 	@property
 	def description(self):
-		return _best_language_match(self.descriptions)
+		return _best_language_match(self.descriptions) or self.first_description
 
 class DummyFeed(object):
 	"""Temporary class used during API transition."""
@@ -1057,7 +1176,13 @@ def _pretty_escape(uri):
 	: is preserved and / becomes #. This makes for nicer strings,
 	and may replace L{escape} everywhere in future.
 	@rtype: str"""
-	return re.sub('[^-_.a-zA-Z0-9:/]',
+	if os.name == "posix":
+		# Only preserve : on Posix systems
+		preserveRegex = '[^-_.a-zA-Z0-9:/]'
+	else:
+		# Other OSes may not allow the : character in file names
+		preserveRegex = '[^-_.a-zA-Z0-9/]'
+	return re.sub(preserveRegex,
 		lambda match: '%%%02x' % ord(match.group(0)),
 		uri.encode('utf-8')).replace('/', '#')
 
@@ -1136,9 +1261,9 @@ def parse_version(version_string):
 		for x in range(1, l, 2):
 			parts[x] = _version_mod_to_value[parts[x]]
 		return parts
-	except ValueError, ex:
+	except ValueError as ex:
 		raise SafeException(_("Invalid version format in '%(version_string)s': %(exception)s") % {'version_string': version_string, 'exception': ex})
-	except KeyError, ex:
+	except KeyError as ex:
 		raise SafeException(_("Invalid version modifier in '%(version_string)s': %(exception)s") % {'version_string': version_string, 'exception': ex})
 
 def format_version(version):
