@@ -6,36 +6,13 @@ Chooses a set of components to make a running program.
 # See the README file for details, or visit http://0install.net.
 
 from zeroinstall import _
-import os, tempfile, subprocess, sys, locale
+import locale
 from logging import debug, warn, info
 
-from zeroinstall.zerostore import BadDigest, NotStored
-
+from zeroinstall.injector.reader import MissingLocalFeed
 from zeroinstall.injector.arch import machine_groups
 from zeroinstall.injector import model, sat, selections
 
-def _get_cached(stores, impl):
-	"""Check whether an implementation is available locally.
-	@type impl: model.Implementation
-	@rtype: bool
-	"""
-	if isinstance(impl, model.DistributionImplementation):
-		return impl.installed
-	if impl.local_path:
-		return os.path.exists(impl.local_path)
-	else:
-		try:
-			if not impl.digests:
-				warn("No digests given for %s!", impl)
-				return False
-			path = stores.lookup_any(impl.digests)
-			assert path
-			return True
-		except BadDigest:
-			return False
-		except NotStored:
-			return False
-
 class CommandInfo:
 	def __init__(self, name, command, impl, arch):
 		self.name = name
@@ -110,36 +87,56 @@ class Solver(object):
 		@param root_arch: the desired target architecture
 		@type root_arch: L{arch.Architecture}
 		@param command_name: which <command> element to select
-		@type command_name: str
+		@type command_name: str | None
 		@postcondition: self.ready, self.selections and self.feeds_used are updated"""
 		raise NotImplementedError("Abstract")
 
 class SATSolver(Solver):
-	__slots__ = ['_failure_reason', 'network_use', 'iface_cache', 'stores', 'help_with_testing', 'extra_restrictions',
-			'langs']
-
 	"""Converts the problem to a set of pseudo-boolean constraints and uses a PB solver to solve them.
 	@ivar langs: the preferred languages (e.g. ["es_ES", "en"]). Initialised to the current locale.
 	@type langs: str"""
-	def __init__(self, network_use, iface_cache, stores, extra_restrictions = None):
+
+	__slots__ = ['_failure_reason', 'config', 'extra_restrictions', '_lang_ranks', '_langs']
+
+	@property
+	def iface_cache(self):
+		return self.config.iface_cache	# (deprecated; used by 0compile)
+
+	def __init__(self, config, extra_restrictions = None):
 		"""
-		@param network_use: how much use to make of the network
-		@type network_use: L{model.network_levels}
-		@param iface_cache: a cache of feeds containing information about available versions
-		@type iface_cache: L{iface_cache.IfaceCache}
-		@param stores: a cached of implementations (affects choice when offline or when minimising network use)
-		@type stores: L{zerostore.Stores}
+		@param config: policy preferences (e.g. stability), the iface_cache and the stores to use
+		@type config: L{policy.Config}
 		@param extra_restrictions: extra restrictions on the chosen implementations
 		@type extra_restrictions: {L{model.Interface}: [L{model.Restriction}]}
 		"""
 		Solver.__init__(self)
-		self.network_use = network_use
-		self.iface_cache = iface_cache
-		self.stores = stores
-		self.help_with_testing = False
+		assert not isinstance(config, str), "API change!"
+		self.config = config
 		self.extra_restrictions = extra_restrictions or {}
 
-		self.langs = [locale.getlocale()[0] or 'en']
+		# By default, prefer the current locale's language first and English second
+		self.langs = [locale.getlocale()[0] or 'en', 'en']
+
+	def set_langs(self, langs):
+		"""Set the preferred languages.
+		@param lang: languages (and regions), first choice first
+		@type lang: [str]
+		"""
+		# _lang_ranks is a map from locale string to score (higher is better)
+		_lang_ranks = {}
+		score = 0
+		i = len(langs)
+		# (is there are duplicates, the first occurance takes precedence)
+		while i > 0:
+			i -= 1
+			lang = langs[i].replace('_', '-')
+			_lang_ranks[lang.split('-')[0]] = score
+			_lang_ranks[lang] = score + 1
+			score += 2
+		self._langs = langs
+		self._lang_ranks = _lang_ranks
+
+	langs = property(lambda self: self._langs, set_langs)
 
 	def compare(self, interface, b, a, arch):
 		"""Compare a and b to see which would be chosen first.
@@ -151,9 +148,10 @@ class SATSolver(Solver):
 		# Languages we understand come first
 		a_langs = (a.langs or 'en').split()
 		b_langs = (b.langs or 'en').split()
-		main_langs = set(l.split('_')[0] for l in self.langs)
-		r = cmp(any(l.split('_')[0] in main_langs for l in a_langs),
-			any(l.split('_')[0] in main_langs for l in b_langs))
+		my_langs = self._lang_ranks
+
+		r = cmp(max(my_langs.get(l.split('-')[0], -1) for l in a_langs),
+			max(my_langs.get(l.split('-')[0], -1) for l in b_langs))
 		if r: return r
 
 		a_stab = a.get_stability()
@@ -163,9 +161,9 @@ class SATSolver(Solver):
 		r = cmp(a_stab == model.preferred, b_stab == model.preferred)
 		if r: return r
 
-		stores = self.stores
-		if self.network_use != model.network_full:
-			r = cmp(_get_cached(stores, a), _get_cached(stores, b))
+		stores = self.config.stores
+		if self.config.network_use != model.network_full:
+			r = cmp(a.is_available(stores), b.is_available(stores))
 			if r: return r
 
 		# Packages that require admin access to install come last
@@ -175,7 +173,7 @@ class SATSolver(Solver):
 		# Stability
 		stab_policy = interface.stability_policy
 		if not stab_policy:
-			if self.help_with_testing: stab_policy = model.testing
+			if self.config.help_with_testing: stab_policy = model.testing
 			else: stab_policy = model.stable
 
 		if a_stab >= stab_policy: a_stab = model.preferred
@@ -207,13 +205,14 @@ class SATSolver(Solver):
 		if r: return r
 
 		# Slightly prefer languages specialised to our country
-		r = cmp(any(l in self.langs for l in a_langs),
-			any(l in self.langs for l in b_langs))
+		# (we know a and b have the same base language at this point)
+		r = cmp(max(my_langs.get(l, -1) for l in a_langs),
+			max(my_langs.get(l, -1) for l in b_langs))
 		if r: return r
 
 		# Slightly prefer cached versions
-		if self.network_use == model.network_full:
-			r = cmp(_get_cached(stores, a), _get_cached(stores, b))
+		if self.config.network_use == model.network_full:
+			r = cmp(a.is_available(stores), b.is_available(stores))
 			if r: return r
 
 		return cmp(a.id, b.id)
@@ -223,13 +222,25 @@ class SATSolver(Solver):
 		# by valid implementation to every interface, so we can always
 		# select something. Useful for diagnostics.
 
+		# The basic plan is this:
+		# 1. Scan the root interface and all dependencies recursively, building up a SAT problem.
+		# 2. Solve the SAT problem. Whenever there are multiple options, try the most preferred one first.
+		# 3. Create a Selections object from the results.
+		#
+		# All three involve recursively walking the tree in a similar way:
+		# 1) we follow every dependency of every implementation (order not important)
+		# 2) we follow every dependency of every selected implementation (better versions first)
+		# 3) we follow every dependency of every selected implementation (order doesn't matter)
+		#
+		# In all cases, a dependency may be on an <implementation> or on a specific <command>.
+
 		# TODO: We need some way to figure out which feeds to include.
 		# Currently, we include any feed referenced from anywhere but
 		# this is probably too much. We could insert a dummy optimial
 		# implementation in stale/uncached feeds and see whether it
 		# selects that.
+		iface_cache = self.config.iface_cache
 
-		feeds_added = set()
 		problem = sat.SATProblem()
 
 		impl_to_var = {}	# Impl -> sat var
@@ -262,26 +273,46 @@ class SATSolver(Solver):
 					continue
 				yield dep
 
-		# Add a clause so that if requiring_impl_var is True then an implementation
-		# matching 'dependency' must also be selected.
 		# Must have already done add_iface on dependency.interface.
+		# If dependency is essential:
+		#   Add a clause so that if requiring_impl_var is True then an implementation
+		#   matching 'dependency' must also be selected.
+		# If dependency is optional:
+		#   Require that no incompatible version is selected.
+		# This ignores any 'command' required. Handle that separately.
 		def find_dependency_candidates(requiring_impl_var, dependency):
-			dep_iface = self.iface_cache.get_interface(dependency.interface)
+			def meets_restrictions(candidate):
+				for r in dependency.restrictions:
+					if not r.meets_restriction(candidate):
+						#warn("%s rejected due to %s", candidate.get_version(), r)
+						return False
+				return True
+
+			essential = dependency.importance == model.Dependency.Essential
+
+			dep_iface = iface_cache.get_interface(dependency.interface)
 			dep_union = [sat.neg(requiring_impl_var)]	# Either requiring_impl_var is False, or ...
 			for candidate in impls_for_iface[dep_iface]:
-				for r in dependency.restrictions:
-					if candidate.__class__ is not _DummyImpl and not r.meets_restriction(candidate):
-						#warn("%s rejected due to %s", candidate.get_version(), r)
-						if candidate.version is not None:
-							break
+				if (candidate.__class__ is _DummyImpl) or meets_restrictions(candidate):
+					if essential:
+						c_var = impl_to_var.get(candidate, None)
+						if c_var is not None:
+							dep_union.append(c_var)
+						# else we filtered that version out, so ignore it
 				else:
-					c_var = impl_to_var.get(candidate, None)
-					if c_var is not None:
-						dep_union.append(c_var)
-					# else we filtered that version out, so ignore it
+					# Candidate doesn't meet our requirements
+					# If the dependency is optional add a rule to make sure we don't
+					# select this candidate.
+					# (for essential dependencies this isn't necessary because we must
+					# select a good version and we can't select two)
+					if not essential:
+						c_var = impl_to_var.get(candidate, None)
+						if c_var is not None:
+							problem.add_clause(dep_union + [sat.neg(c_var)])
+						# else we filtered that version out, so ignore it
 
-			assert dep_union
-			problem.add_clause(dep_union)
+			if essential:
+				problem.add_clause(dep_union)
 
 		def is_unusable(impl, restrictions, arch):
 			"""@return: whether this implementation is unusable.
@@ -302,7 +333,7 @@ class SATSolver(Solver):
 			stability = impl.get_stability()
 			if stability <= model.buggy:
 				return stability.name
-			if (self.network_use == model.network_offline or not impl.download_sources) and not _get_cached(self.stores, impl):
+			if (self.config.network_use == model.network_offline or not impl.download_sources) and not impl.is_available(self.config.stores):
 				if not impl.download_sources:
 					return _("No retrieval methods")
 				return _("Not cached and we are off-line")
@@ -319,7 +350,7 @@ class SATSolver(Solver):
 			@rtype: generator(ZeroInstallFeed)"""
 			yield iface.uri
 
-			for f in self.iface_cache.get_feed_imports(iface):
+			for f in iface_cache.get_feed_imports(iface):
 				# Note: when searching for src, None is not in machine_ranks
 				if f.os in arch.os_ranks and \
 				   (f.machine is None or f.machine in arch.machine_ranks):
@@ -328,13 +359,31 @@ class SATSolver(Solver):
 					debug(_("Skipping '%(feed)s'; unsupported architecture %(os)s-%(machine)s"),
 						{'feed': f, 'os': f.os, 'machine': f.machine})
 
+		# If requiring_var is True then all of requirer's dependencies must be satisfied.
+		# requirer can be a <command> or an <implementation>
+		def process_dependencies(requiring_var, requirer, arch):
+			for d in deps_in_use(requirer, arch):
+				debug(_("Considering command dependency %s"), d)
+
+				add_iface(d.interface, arch.child_arch)
+
+				for c in d.get_required_commands():
+					# We depend on a specific command within the implementation.
+					command_vars = add_command_iface(d.interface, arch.child_arch, c)
+
+					# If the parent command/impl is chosen, one of the candidate commands
+					# must be too. If there aren't any, then this command is unselectable.
+					problem.add_clause([sat.neg(requiring_var)] + command_vars)
+
+				# Must choose one version of d if impl is selected
+				find_dependency_candidates(requiring_var, d)
+
 		def add_iface(uri, arch):
 			"""Name implementations from feed and assert that only one can be selected."""
 			if uri in ifaces_processed: return
 			ifaces_processed.add(uri)
-			iface_name = 'i%d' % len(ifaces_processed)
 
-			iface = self.iface_cache.get_interface(uri)
+			iface = iface_cache.get_interface(uri)
 
 			impls = []
 			for f in usable_feeds(iface, arch):
@@ -342,7 +391,7 @@ class SATSolver(Solver):
 				debug(_("Processing feed %s"), f)
 
 				try:
-					feed = self.iface_cache.get_feed(f)
+					feed = iface_cache.get_feed(f)
 					if feed is None: continue
 					#if feed.name and iface.uri != feed.url and iface.uri not in feed.feed_for:
 					#	info(_("Missing <feed-for> for '%(uri)s' in '%(feed)s'"), {'uri': iface.uri, 'feed': f})
@@ -353,11 +402,16 @@ class SATSolver(Solver):
 					distro_feed_url = feed.get_distro_feed()
 					if distro_feed_url:
 						self.feeds_used.add(distro_feed_url)
-						distro_feed = self.iface_cache.get_feed(distro_feed_url)
+						distro_feed = iface_cache.get_feed(distro_feed_url)
 						if distro_feed.implementations:
 							impls.extend(distro_feed.implementations.values())
-				except Exception, ex:
+				except MissingLocalFeed as ex:
+					warn(_("Missing local feed; if it's no longer required, remove it with:") +
+							'\n0install remove-feed ' + iface.uri + ' ' + f,
+						{'feed': f, 'interface': iface, 'exception': ex})
+				except Exception as ex:
 					warn(_("Failed to load feed %(feed)s for %(interface)s: %(exception)s"), {'feed': f, 'interface': iface, 'exception': ex})
+					#raise
 
 			impls.sort(lambda a, b: self.compare(iface, a, b, arch))
 
@@ -385,13 +439,7 @@ class SATSolver(Solver):
 				if impl.machine and impl.machine != 'src':
 					impls_for_machine_group[machine_groups.get(impl.machine, 0)].append(v)
 
-				for d in deps_in_use(impl, arch):
-					debug(_("Considering dependency %s"), d)
-
-					add_iface(d.interface, arch.child_arch)
-
-					# Must choose one version of d if impl is selected
-					find_dependency_candidates(v, d)
+				process_dependencies(v, impl, arch)
 
 			if closest_match:
 				dummy_impl = _DummyImpl()
@@ -423,22 +471,29 @@ class SATSolver(Solver):
 		def add_command_iface(uri, arch, command_name):
 			"""Add every <command> in interface 'uri' with this name.
 			Each one depends on the corresponding implementation and only
-			one can be selected."""
+			one can be selected. If closest_match is on, include a dummy
+			command that can always be selected."""
+
+			# Check whether we've already processed this (interface,command) pair
+			existing = group_clause_for_command.get((uri, command_name), None)
+			if existing is not None:
+				return existing.lits
 
 			# First ensure that the interface itself has been processed
 			# We'll reuse the ordering of the implementations to order
 			# the commands too.
 			add_iface(uri, arch)
 
-			iface = self.iface_cache.get_interface(uri)
+			iface = iface_cache.get_interface(uri)
 			filtered_impls = impls_for_iface[iface]
 
 			var_names = []
 			for impl in filtered_impls:
 				command = impl.commands.get(command_name, None)
 				if not command:
-					# Mark implementation as unselectable
-					problem.add_clause([sat.neg(impl_to_var[impl])])
+					if not isinstance(impl, _DummyImpl):
+						# Mark implementation as unselectable
+						problem.add_clause([sat.neg(impl_to_var[impl])])
 					continue
 
 				# We have a candidate <command>. Require that if it's selected
@@ -448,13 +503,7 @@ class SATSolver(Solver):
 
 				var_names.append(command_var)
 
-				for d in deps_in_use(command, arch):
-					debug(_("Considering command dependency %s"), d)
-
-					add_iface(d.interface, arch.child_arch)
-
-					# Must choose one version of d if impl is selected
-					find_dependency_candidates(command_var, d)
+				process_dependencies(command_var, command, arch)
 
 			# Tell the user why we couldn't use this version
 			if self.record_details:
@@ -464,26 +513,41 @@ class SATSolver(Solver):
 					return old_reason or (_('No %s command') % command_name)
 				self.details[iface] = [(impl, new_reason(impl, reason)) for (impl, reason) in self.details[iface]]
 
+			if closest_match:
+				dummy_command = problem.add_variable(None)
+				var_names.append(dummy_command)
+
+			if var_names:
+				# Can't select more than one of them.
+				assert (uri, command_name) not in group_clause_for_command
+				group_clause_for_command[(uri, command_name)] = problem.at_most_one(var_names)
+
 			return var_names
 
-		commands = add_command_iface(root_interface, root_arch, command_name)
-		if commands:
-			problem.add_clause(commands)		# At least one
-			group_clause_for_command[(root_interface, command_name)] = problem.at_most_one(commands)
+		if command_name is None:
+			add_iface(root_interface, root_arch)
 		else:
-			# (note: might be because we haven't cached it yet)
-			info("No %s <command> in %s", command_name, root_interface)
-
-			impls = impls_for_iface[self.iface_cache.get_interface(root_interface)]
-			if impls == [] or (len(impls) == 1 and isinstance(impls[0], _DummyImpl)):
-				# There were no candidates at all.
-				self._failure_reason = _("Interface '%s' has no usable implementations") % root_interface
+			commands = add_command_iface(root_interface, root_arch, command_name)
+			if len(commands) > int(closest_match):
+				# (we have at least one non-dummy command)
+				problem.add_clause(commands)		# At least one
 			else:
-				# We had some candidates implementations, but none for the command we need
-				self._failure_reason = _("Interface '%s' cannot be executed directly; it is just a library "
-					    "to be used by other programs (or missing '%s' command)") % (root_interface, command_name)
+				# (note: might be because we haven't cached it yet)
+				info("No %s <command> in %s", command_name, root_interface)
 
-			problem.impossible()
+				impls = impls_for_iface[iface_cache.get_interface(root_interface)]
+				if impls == [] or (len(impls) == 1 and isinstance(impls[0], _DummyImpl)):
+					# There were no candidates at all.
+					if self.config.network_use == model.network_offline:
+						self._failure_reason = _("Interface '%s' has no usable implementations in the cache (and 0install is in off-line mode)") % root_interface
+					else:
+						self._failure_reason = _("Interface '%s' has no usable implementations") % root_interface
+				else:
+					# We had some candidates implementations, but none for the command we need
+					self._failure_reason = _("Interface '%s' cannot be executed directly; it is just a library "
+						    "to be used by other programs (or missing '%s' command)") % (root_interface, command_name)
+
+				problem.impossible()
 
 		# Require m<group> to be true if we select an implementation in that group
 		m_groups = []
@@ -506,6 +570,10 @@ class SATSolver(Solver):
 			def find_undecided_dep(impl_or_command, arch):
 				# Check for undecided dependencies of impl_or_command
 				for dep in deps_in_use(impl_or_command, arch):
+					for c in dep.get_required_commands():
+						dep_lit = find_undecided_command(dep.interface, c)
+						if dep_lit is not None:
+							return dep_lit
 					dep_lit = find_undecided(dep.interface)
 					if dep_lit is not None:
 						return dep_lit
@@ -540,6 +608,9 @@ class SATSolver(Solver):
 				# Check for undecided command-specific dependencies, and then for
 				# implementation dependencies.
 				lit_info = problem.get_varinfo_for_lit(lit).obj
+				if lit_info is None:
+					assert closest_match
+					return None	# (a dummy command added for better diagnostics; has no dependencies)
 				return find_undecided_dep(lit_info.command, lit_info.arch) or \
 				       find_undecided_dep(lit_info.impl, lit_info.arch)
 
@@ -549,7 +620,7 @@ class SATSolver(Solver):
 
 			# If we're chosen everything we need, we can probably
 			# set everything else to False.
-			for group in group_clause_for.values() + [m_groups_clause]:
+			for group in group_clause_for.values() + group_clause_for_command.values() + [m_groups_clause]:
 				if group.current is None:
 					best = group.best_undecided()
 					if best is not None:
@@ -571,6 +642,8 @@ class SATSolver(Solver):
 
 			sels = self.selections.selections
 
+			commands_needed = []
+
 			for uri, group in group_clause_for.iteritems():
 				if group.current is not None:
 					lit_info = problem.get_varinfo_for_lit(group.current).obj
@@ -582,6 +655,8 @@ class SATSolver(Solver):
 						deps = self.requires[lit_info.iface] = []
 						for dep in deps_in_use(lit_info.impl, lit_info.arch):
 							deps.append(dep)
+							for c in dep.get_required_commands():
+								commands_needed.append((dep.interface, c))
 	
 						sels[lit_info.iface.uri] = selections.ImplSelection(lit_info.iface.uri, impl, deps)
 
@@ -589,13 +664,17 @@ class SATSolver(Solver):
 				sel = sels.get(iface, None)
 				if sel:
 					command = sel.impl.commands[name]
-					self.selections.commands.append(command)
+					sel._used_commands.add(name)
 					runner = command.get_runner()
 					if runner:
-						# TODO: allow depending on other commands, besides 'run'?
-						add_command(runner.metadata['interface'], 'run')
+						add_command(runner.metadata['interface'], runner.command)
 
-			add_command(root_interface, command_name)
+			for iface, command in commands_needed:
+				add_command(iface, command)
+
+			if command_name is not None:
+				self.selections.command = command_name
+				add_command(root_interface, command_name)
 
 	def get_failure_reason(self):
 		"""Return an exception explaining why the solve failed."""
