@@ -39,7 +39,7 @@ callbacks. See the Task class (below) for more information.
 
 from zeroinstall import _
 import sys
-from logging import info, warn
+from logging import info, warn, debug
 import gobject
 
 # The list of Blockers whose event has happened, in the order they were
@@ -137,6 +137,7 @@ class Blocker:
 		"""Called by the schedular when a Task yields this
 		Blocker. If you override this method, be sure to still
 		call this method with Blocker.add_task(self)!"""
+		assert task not in self._zero_lib_tasks, "Blocking on a single task twice: %s (%s)" % (task, self)
 		self._zero_lib_tasks.add(task)
 	
 	def remove_task(self, task):
@@ -164,7 +165,7 @@ class TimeoutBlocker(Blocker):
 	def __init__(self, timeout, name):
 		"""Trigger after 'timeout' seconds (may be a fraction)."""
 		Blocker.__init__(self, name)
-		gobject.timeout_add(long(timeout * 1000), self._timeout)
+		gobject.timeout_add(int(timeout * 1000), self._timeout)
 	
 	def _timeout(self):
 		self.trigger()
@@ -250,7 +251,6 @@ class Task:
 	
 	def _resume(self):
 		# Remove from our blockers' queues
-		exception = None
 		for blocker in self._zero_blockers:
 			blocker.remove_task(self)
 		# Resume the task
@@ -262,7 +262,7 @@ class Task:
 			return
 		except SystemExit:
 			raise
-		except (Exception, KeyboardInterrupt), ex:
+		except (Exception, KeyboardInterrupt) as ex:
 			# Task crashed
 			info(_("Exception from '%(name)s': %(exception)s"), {'name': self.finished.name, 'exception': ex})
 			#import traceback
@@ -350,3 +350,29 @@ def async(fn):
 		return Task(fn(*args, **kwargs), fn.__name__).finished
 	run.__name__ = fn.__name__
 	return run
+
+def wait_for_blocker(blocker):
+	"""Run a recursive mainloop until blocker is triggered.
+	@param blocker: event to wait on
+	@type blocker: L{Blocker}
+	@since: 0.53
+	"""
+	assert wait_for_blocker.loop is None	# Avoid recursion
+
+	if not blocker.happened:
+		def quitter():
+			yield blocker
+			wait_for_blocker.loop.quit()
+		Task(quitter(), "quitter")
+
+		wait_for_blocker.loop = gobject.MainLoop(gobject.main_context_default())
+		try:
+			debug(_("Entering mainloop, waiting for %s"), blocker)
+			wait_for_blocker.loop.run()
+		finally:
+			wait_for_blocker.loop = None
+
+		assert blocker.happened, "Someone quit the main loop!"
+
+	check(blocker)
+wait_for_blocker.loop = None
