@@ -70,11 +70,15 @@ def type_from_url(url):
 	if url.endswith('.tar.gz'): return 'application/x-compressed-tar'
 	if url.endswith('.tar.lzma'): return 'application/x-lzma-compressed-tar'
 	if url.endswith('.tar.xz'): return 'application/x-xz-compressed-tar'
+	if url.endswith('.tbz'): return 'application/x-bzip-compressed-tar'
 	if url.endswith('.tgz'): return 'application/x-compressed-tar'
+	if url.endswith('.tlz'): return 'application/x-lzma-compressed-tar'
+	if url.endswith('.txz'): return 'application/x-xz-compressed-tar'
 	if url.endswith('.tar'): return 'application/x-tar'
 	if url.endswith('.zip'): return 'application/zip'
 	if url.endswith('.cab'): return 'application/vnd.ms-cab-compressed'
 	if url.endswith('.dmg'): return 'application/x-apple-diskimage'
+	if url.endswith('.gem'): return 'application/x-ruby-gem'
 	return None
 
 def check_type_ok(mime_type):
@@ -112,7 +116,7 @@ def check_type_ok(mime_type):
 			raise SafeException(_("This package looks like a xz-compressed package, but you don't have the 'unxz' command "
 					"I need to extract it. Install the package containing it (it's probably called 'xz-utils') "
 					"first."))
-	elif mime_type in ('application/x-compressed-tar', 'application/x-tar'):
+	elif mime_type in ('application/x-compressed-tar', 'application/x-tar', 'application/x-ruby-gem'):
 		pass
 	else:
 		from zeroinstall import version
@@ -160,7 +164,7 @@ def unpack_archive_over(url, data, destdir, extract = None, type = None, start_o
 			target_root = os.path.join(destdir, relative_root)
 			try:
 				info = os.lstat(target_root)
-			except OSError, ex:
+			except OSError as ex:
 				if ex.errno != 2:
 					raise	# Some odd error.
 				# Doesn't exist. OK.
@@ -216,6 +220,8 @@ def unpack_archive(url, data, destdir, extract = None, type = None, start_offset
 		extract_cab(data, destdir, extract, start_offset)
 	elif type == 'application/x-apple-diskimage':
 		extract_dmg(data, destdir, extract, start_offset)
+	elif type == 'application/x-ruby-gem':
+		extract_gem(data, destdir, extract, start_offset)
 	else:
 		raise SafeException(_('Unknown MIME type "%(type)s" for "%(url)s"') % {'type': type, 'url': url})
 
@@ -293,6 +299,21 @@ def extract_rpm(stream, destdir, extract = None, start_offset = 0):
 			os.close(fd)
 		os.unlink(cpiopath)
 
+def extract_gem(stream, destdir, extract = None, start_offset = 0):
+	"@since: 0.53"
+	stream.seek(start_offset)
+	payload = 'data.tar.gz'
+	payload_stream = None
+	tmpdir = mkdtemp(dir = destdir)
+	try:
+		extract_tar(stream, destdir=tmpdir, extract=payload, decompress=None)
+		payload_stream = file(os.path.join(tmpdir, payload))
+		extract_tar(payload_stream, destdir=destdir, extract=extract, decompress='gzip')
+	finally:
+		if payload_stream:
+			payload_stream.close()
+		ro_rmtree(tmpdir)
+
 def extract_cab(stream, destdir, extract, start_offset = 0):
 	"@since: 0.24"
 	if extract:
@@ -371,7 +392,10 @@ def extract_tar(stream, destdir, extract, decompress, start_offset = 0):
 					unlzma = os.path.abspath(os.path.join(os.path.dirname(__file__), '_unlzma'))
 				ext_cmd.append('--use-compress-program=' + unlzma)
 			elif decompress == 'xz':
-				ext_cmd.append('--use-compress-program=unxz')
+				unxz = find_in_path('unxz')
+				if not unxz:
+					unxz = os.path.abspath(os.path.join(os.path.dirname(__file__), '_unxz'))
+				ext_cmd.append('--use-compress-program=' + unxz)
 
 		if recent_gnu_tar():
 			ext_cmd.extend(('-x', '--no-same-owner', '--no-same-permissions'))
@@ -383,14 +407,34 @@ def extract_tar(stream, destdir, extract, decompress, start_offset = 0):
 
 		_extract(stream, destdir, ext_cmd, start_offset)
 	else:
+		import tempfile
+
 		# Since we don't have GNU tar, use python's tarfile module. This will probably
 		# be a lot slower and we do not support lzma and xz; however, it is portable.
+		# (lzma and xz are handled by first uncompressing stream to a temporary file.
+		# this is simple to do, but less efficient than piping through the program)
 		if decompress is None:
 			rmode = 'r|'
 		elif decompress == 'bzip2':
 			rmode = 'r|bz2'
 		elif decompress == 'gzip':
 			rmode = 'r|gz'
+		elif decompress == 'lzma':
+			unlzma = find_in_path('unlzma')
+			if not unlzma:
+				unlzma = os.path.abspath(os.path.join(os.path.dirname(__file__), '_unlzma'))
+			temp = tempfile.NamedTemporaryFile(suffix='.tar')
+			subprocess.check_call((unlzma), stdin=stream, stdout=temp)
+			rmode = 'r|'
+			stream = temp
+		elif decompress == 'xz':
+			unxz = find_in_path('unxz')
+			if not unxz:
+				unxz = os.path.abspath(os.path.join(os.path.dirname(__file__), '_unxz'))
+			temp = tempfile.NamedTemporaryFile(suffix='.tar')
+			subprocess.check_call((unxz), stdin=stream, stdout=temp)
+			rmode = 'r|'
+			stream = temp
 		else:
 			raise SafeException(_('GNU tar unavailable; unsupported compression format: %s') % decompress)
 
@@ -412,12 +456,12 @@ def extract_tar(stream, destdir, extract, decompress, start_offset = 0):
 
 		def chmod_extract(tarinfo):
 			# If any X bit is set, they all must be
-			if tarinfo.mode & 0111:
-				tarinfo.mode |= 0111
+			if tarinfo.mode & 0o111:
+				tarinfo.mode |= 0o111
 
 			# Everyone gets read and write (subject to the umask)
 			# No special bits are allowed.
-			tarinfo.mode = ((tarinfo.mode | 0666) & ~current_umask) & 0777
+			tarinfo.mode = ((tarinfo.mode | 0o666) & ~current_umask) & 0o777
 
 			# Don't change owner, even if run as root
 			if uid:
